<?php
declare(strict_types = 1);

/**
 * Gestion du diaporama.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
class AjaxDiaporama extends Ajax
{
	/**
	 * Informations utiles de la catégorie.
	 *
	 * @var array
	 */
	private static $_categoryInfos = [];

	/**
	 * Nombre de fichiers de la série.
	 *
	 * @var int
	 */
	private static $_count = 0;

	/**
	 * Paramètres de la requête SQL.
	 *
	 * @var array
	 */
	private static $_params = [];

	/**
	 * Clause FROM de la requête SQL.
	 *
	 * @var string
	 */
	private static $_sqlFrom = "";

	/**
	 * Clause ORDER BY de la requête SQL.
	 *
	 * @var string
	 */
	private static $_sqlOrderBy = "";

	/**
	 * Clause WHERE de la requête SQL.
	 *
	 * @var string
	 */
	private static $_sqlWhere = "";



	/**
	 * Récupération des données pour le diaporama.
	 *
	 * @return void
	 */
	public static function getData(): void
	{
		if (!Config::$params['diaporama'] || !isset($_POST['q'])
		|| !isset($_POST['carousel_position'])
		|| !preg_match('`^\d{1,12}$`', (string) $_POST['carousel_position'])
		|| !isset($_POST['get_carousel'])
		|| !isset($_POST['get_item'])
		|| !isset($_POST['item_position'])
		|| !preg_match('`^\d{1,12}$`', (string) $_POST['item_position'])
		|| !isset($_POST['item_preload'])
		|| !preg_match('`^\d{1,2}$`', (string) $_POST['item_preload']))
		{
			self::_forbidden();
		}

		// Requête.
		$_GET['q'] = $_POST['q'] ?? '';
		self::_request();

		// Filtres.
		App::filtersGET();

		// Informations de la catégorie.
		self::_getCategoryInfos();

		$_POST['preload'] = $_POST['preload'] ?? 2;

		// Clause WHERE : permissions.
		self::$_sqlWhere = "item_status = '1' AND "
			. SQL::catPerms() . " AND " . SQL::catPassword();

		// Clause WHERE : filtre.
		if (isset($_GET['filter']))
		{
			$search_where = function()
			{
				$search = new Search();
				if ($search->sql($_GET['search']) && !isset(self::$search->options['type']))
				{
					return $search->sql();
				}
				return FALSE;
			};
			if (($_GET['filter'] == 'search' && $where = $search_where())
			|| $where = SQL::itemsWhere($_GET['filter'], $_GET['filter_value']))
			{
				self::$_sqlWhere .= " AND " . $where['sql'];
				self::$_params = $where['params'];
			}
			else
			{
				self::_forbidden();
			}
		}

		// Clause WHERE : catégorie.
		if (self::$_categoryInfos['cat_id'] > 1)
		{
			self::$_sqlWhere .= " AND item_path LIKE :path";
			self::$_params['path'] = DB::likeEscape(self::$_categoryInfos['cat_path']) . '/%';
		}

		// Clause FROM.
		self::$_sqlFrom = "{items} AS i "
			. SQL::itemsFrom($_GET['filter'] ?? '')
			. " LEFT JOIN {categories} AS cat ON i.cat_id = cat.cat_id";

		// Clause ORDER BY.
		self::$_sqlOrderBy = SQL::itemsOrderBy(
			$_GET['filter'] ?? '',
			self::$_categoryInfos['cat_orderby'] ?? Config::$params['items_sql_order_by'],
			self::$_categoryInfos
		);

		// Nombre de fichiers dans la série.
		$sql = "SELECT COUNT(*)
				  FROM " . self::$_sqlFrom . "
				 WHERE " . self::$_sqlWhere;
		if (!DB::execute($sql, self::$_params))
		{
			self::_dbError();
		}
		self::$_count = DB::fetchVal();

		// Carrousel.
		if ($_POST['get_carousel'])
		{
			$carousel = self::_getCarousel();
		}

		// Fichiers.
		if ($_POST['get_item'])
		{
			$items = self::_getAllItems();

			// Nombre de vues.
			if ((!isset($_GET['filter']) || $_GET['filter'] != 'views')
			&& Item::hit((int) $items[$_POST['item_position']]['id']) === 1)
			{
				$views = (int) $items[$_POST['item_position']]['stats']['views']['value'] + 1;
				$items[$_POST['item_position']]['stats']['views']['short']
					= L10N::formatShortNumber($views);
				$items[$_POST['item_position']]['stats']['views']['value']
					= $views;
			}
		}

		self::_printResponse(
		[
			'status' => 'success',
			'carousel' => $carousel ?? [],
			'count' => (int) self::$_count,
			'items' => $items ?? []
		]);
	}



	/**
	 * Récupération des informations utiles de tous les fichiers demandés.
	 *
	 * @return array
	 */
	private static function _getAllItems(): array
	{
		// Récupération du fichier actuel et des fichiers précédents et suivants.
		$sql_limit = self::_sqlLimit((int) $_POST['item_position'], (int) $_POST['item_preload']);
		$items = self::_getItems($sql_limit);

		$all_items = [];

		$sql_limit = explode(',', $sql_limit);
		if (is_array($items))
		{
			$n = $sql_limit[0] + 1;
			foreach ($items as &$i)
			{
				$all_items[$n] = $i;
				$n++;
			}
		}

		// Récupération des premiers fichiers de la série.
		if ($sql_limit[0] != '0')
		{
			$items = self::_getItems("0," . $_POST['preload']);

			$n = 1;
			foreach ($items as &$i)
			{
				$all_items[$n] = $i;
				$n++;
			}
		}

		// Récupération des derniers fichiers de la série.
		if (($sql_limit[0] + $sql_limit[1]) < self::$_count)
		{
			$limit = $_POST['preload'];
			$items = self::_getItems((self::$_count - $limit) . "," . $limit);

			$n = self::$_count - $limit + 1;
			foreach ($items as &$i)
			{
				$all_items[$n] = $i;
				$n++;
			}
		}

		// Formatage des données.
		require_once(__DIR__ . '/GalleryItems.class.php');
		require_once(GALLERY_ROOT . '/template/'
			. Config::$params['theme_template'] . '/_config.php');
		$formated = [];
		$inames =
		[
			'date_added', 'date_added_text', 'date_created', 'date_created_text', 'description',
			'duration_text', 'exif', 'height', 'id', 'is_image', 'is_video', 'link', 'owner',
			'source', 'stats', 'title', 'type_mime', 'type_text', 'url', 'user', 'width'
		];
		foreach ($all_items as $k => &$i)
		{
			foreach ($formated_infos = GalleryItems::getFormatedInfos($i) as $fk => &$f)
			{
				if (in_array($fk, $inames))
				{
					$formated[$k][$fk] = $f;
				}
				if ($fk == 'filename' && $formated_infos['user']['edit'])
				{
					$formated[$k][$fk] = $f;
				}
			}

			// Dimensions redimensionnées.
			$formated[$k]['width_resized'] = (int) $i['item_width'];
			$formated[$k]['height_resized'] = (int) $i['item_height'];
			if ($formated[$k]['is_video'])
			{
				if ($formated[$k]['width_resized'] && $formated[$k]['height_resized'])
				{
					$min_width = 800;
					$min_height = 600;
					$resized =
					[
						'w' => $formated[$k]['width_resized'],
						'h' => $formated[$k]['height_resized']
					];

					$ratio_width = $formated[$k]['width_resized'] / $min_width;
					$ratio_height = $formated[$k]['height_resized'] / $min_height;

					if (($formated[$k]['width_resized'] < $min_width)
					&& ($ratio_width >= $ratio_height))
					{
						$resized['w'] = $min_width;
						$resized['h'] = round($formated[$k]['height_resized'] / $ratio_width);
					}
					else if (($formated[$k]['height_resized'] < $min_height)
					&& ($ratio_height >= $ratio_width))
					{
						$resized['w'] = round($formated[$k]['width_resized'] / $ratio_height);
						$resized['h'] = $min_height;
					}

					$formated[$k]['width'] = $formated[$k]['width_resized'] = $resized['w'];
					$formated[$k]['height'] = $formated[$k]['height_resized'] = $resized['h'];
				}
				else
				{
					$formated[$k]['width_resized'] = 800;
					$formated[$k]['height_resized'] = 600;
				}
			}
			elseif (preg_match('`resize\.php.+&size=(\d{1,5})x(\d{1,5})`',
			$formated[$k]['source'], $m))
			{
				$resized = Image::getResizedSize((int) $i['item_width'],
					(int) $i['item_height'], (int) $m[1], (int) $m[2]);
				$formated[$k]['width_resized'] = $resized['w'];
				$formated[$k]['height_resized'] = $resized['h'];
			}

			// Position du fichier dans la série.
			$text = $formated[$k]['is_video'] ? __('Vidéo %s/%s') : __('Photo %s/%s');
			$position_text = sprintf($text, $k, self::$_count);
			$formated[$k]['position_text'] = $position_text;
			$formated[$k]['position'] = sprintf('%s/%s', $k, self::$_count);

			// Signature pour identifier le fichier afin de mettre à jour
			// le diaporama en cas de changement du fichier ou de la série.
			$formated[$k]['md5'] = md5(implode('|', [$position_text, $i['item_id'],
				$formated[$k]['height_resized'], $formated[$k]['width_resized'] ]));

			// Poster.
			if ($formated[$k]['is_video'])
			{
				$formated[$k]['poster'] = App::getThumbSource('item', $i,
					(string) $theme_config['diaporama_video_poster_size'], '', TRUE);
			}

			// Téléchargement.
			if (GalleryItems::itemDownload($i))
			{
				$formated[$k]['download'] = App::getFileSource($i['item_path'], TRUE);
			}

			// Statistiques.
			unset($formated[$k]['stats']['filter']);

			// Métadonnées EXIF.
			$formated[$k]['exif'] = GalleryItems::getExif($i);

			// Tags.
			$formated[$k]['tags'] = GalleryItems::getTags($i);

			// Sécurisation des données.
			HTML::specialchars($formated[$k]);

			// Fil d'Ariane.
			Template::breadcrumb($i, TRUE, 'category', FALSE);
			HTML::specialchars(Template::$data['breadcrumb']);
			$breadcrumb = [];
			foreach (Template::$data['breadcrumb'] as $cat)
			{
				$class = in_array($cat['type'], ['image', 'video']) ? ' class="current"' : '';
				$breadcrumb[] = "<a$class href=\"{$cat['url']}\">{$cat['name']}</a>";
			}
			$formated[$k]['breadcrumb'] = implode('<span> / </span>', $breadcrumb);
			unset(Template::$data['breadcrumb']);

			// Description.
			if (is_string($formated[$k]['description']))
			{
				$formated[$k]['description']
					= Template::formatDescription($formated[$k]['description']);
			}

			// Utilisateur.
			if (Config::$params['users'])
			{
				$html = '<span id="diaporama_owner_avatar">';
				if (Auth::$groupPerms['members_profile'])
				{
					$html .= '<a href="' . $formated[$k]['owner']['link'] . '">';
				}
				$html .= '<img alt="' . __('Avatar') . '" width="50" height="50" src="'
					. $formated[$k]['owner']['avatar_source'] . '">';
				if (Auth::$groupPerms['members_profile'])
				{
					$html .= '</a>';
				}
				$html .= '</span>';
				$nickname = $formated[$k]['owner']['nickname'];
				if (Auth::$groupPerms['members_profile'])
				{
					$url = App::getURL('user/' . $formated[$k]['owner']['id']);
					$nickname = '<a href="' . $url . '">' . $nickname . '</a>';
				}
				$date = '<a href="' . App::getURL('category/1/date-added/'
					. $formated[$k]['date_added']) . '">'
					. $formated[$k]['date_added_text'] . '</a>';
				$nickname = '<span id="diaporama_owner_name">' . $nickname . '</span>';
				$html .= '<p>' . sprintf(__('Ajoutée le %s par %s'), $date, $nickname) . '</p>';
				$formated[$k]['owner'] = $html;
			}
			else
			{
				unset($formated[$k]['owner']);
			}
		}

		return $formated;
	}

	/**
	 * Récupération des informations utiles des fichiers du Carrousel.
	 *
	 * @return array
	 */
	private static function _getCarousel(): array
	{
		// Clause LIMIT.
		$sql_limit = self::_sqlLimit((int) $_POST['carousel_position'], 60);

		// On récupère les informations utiles des fichiers.
		$sql = "SELECT i.item_id,
					   i.item_type,
					   i.item_path,
					   i.item_url,
					   i.item_width,
					   i.item_height,
					   i.item_duration,
					   i.item_name,
					   i.item_tb_params,
					   i.item_adddt
				  FROM " . self::$_sqlFrom . "
				 WHERE " . self::$_sqlWhere . "
			  ORDER BY " . self::$_sqlOrderBy . "
			     LIMIT $sql_limit";
		if (!DB::execute($sql, self::$_params))
		{
			self::_dbError();
		}

		$formated = [];
		$position = (int) (preg_replace('`,\d+$`', '', $sql_limit)) + 1;
		foreach (DB::fetchAll() as &$i)
		{
			$formated[$position] =
			[
				'duration' => $i['item_duration']
					? App::formatDuration($i['item_duration'])
					: '00:00',
				'source' => App::getThumbSource('item', $i),
				'is_video' => Item::isVideo($i['item_type']),
			];
			$position++;
		}

		return $formated;
	}

	/**
	 * Récupération des informations utiles de la catégorie courante.
	 *
	 * @return void
	 */
	private static function _getCategoryInfos(): void
	{
		$sql_type = ($_GET['section'] == 'category')
			? " AND cat_filemtime IS NULL "
			: " AND cat_filemtime IS NOT NULL ";

		$sql = "SELECT cat_id,
					   cat_path,
					   cat_name,
					   cat_url,
					   cat_filemtime,
					   cat_orderby
				  FROM {categories} AS cat
				 WHERE %s
				   AND %s
				   AND cat_id = ?
				   AND cat_status = '1'"
					 . $sql_type;
		$sql = sprintf($sql, SQL::catPerms(), SQL::catPassword());
		if (!DB::execute($sql, (int) $_GET['category_id']))
		{
			self::_dbError();
		}
		if (!isset((self::$_categoryInfos = DB::fetchRow())['cat_id']))
		{
			self::_forbidden();
		}
	}

	/**
	 * Récupération de plusieurs fichiers au sein de la série.
	 *
	 * @return array
	 */
	private static function _getItems(string $sql_limit): array
	{
		$sql_select = "";

		// Favoris.
		if (Config::$params['users'] && Auth::$connected)
		{
			$sql_select .= ", (SELECT 1
						  FROM {favorites}
						 WHERE item_id = i.item_id
						   AND user_id = :user_id
						 LIMIT 1) AS in_favorites";
			self::$_params['user_id'] = Auth::$id;
		}

		// On récupère les informations utiles des fichiers.
		$sql_cameras = "";
		if (empty($_GET['filter']) || substr($_GET['filter'], 0, 6) != 'camera')
		{
			$sql_cameras = "
			 LEFT JOIN {cameras_items} AS cam_i
				    ON cam_i.item_id = i.item_id
		     LEFT JOIN {cameras_models} AS cam_m
				    ON cam_m.camera_model_id = cam_i.camera_model_id
		     LEFT JOIN {cameras_brands} AS cam_b
				    ON cam_b.camera_brand_id = cam_m.camera_brand_id";
		}
		$sql = "SELECT i.*,
					   cat.cat_id,
					   cat.cat_name,
					   cat.cat_url,
					   cat.cat_watermark,
					   cat.cat_parents,
					   'album' AS cat_type,
					   u.user_avatar,
					   u.user_login,
					   u.user_nickname,
					   u.user_status,
					   cam_m.*,
					   cam_b.*
					   $sql_select
				  FROM " . self::$_sqlFrom . "
			 LEFT JOIN {users} AS u
					ON i.user_id = u.user_id
					   $sql_cameras
				 WHERE " . self::$_sqlWhere . "
			  ORDER BY " . self::$_sqlOrderBy . "
			     LIMIT $sql_limit";
		if (!DB::execute($sql, self::$_params))
		{
			self::_dbError();
		}
		$items = DB::fetchAll();

		// Réglages.
		foreach ($items as &$i)
		{
			$cat_parents = $i['cat_parents'];
			$i['cat_parents'] .= $i['cat_id'] . Category::PARENTS_SEPARATOR;
			$i = [$i];
			Parents::settings($i);
			$i = $i[0];
			$i['cat_parents'] = $cat_parents;
		}

		return $items;
	}

	/**
	 * Gestion des paramètres GET.
	 *
	 * @return void
	 */
	private static function _request(): void
	{
		// Configuration de base.
		$request =
		[
			'{category}/{id}',
			'{category}/{id}/camera-brand/{id}',
			'{category}/{id}/camera-model/{id}',
			'{category}/{id}/date-added/{date-flex}',
			'{category}/{id}/date-created/{date-flex}',
			'{category}/{id}/images',
			'{category}/{id}/items',
			'{category}/{id}/videos',
			'{category}/{id}/views'
		];

		// Commentaires.
		if (Config::$params['comments'])
		{
			$request[] = '{category}/{id}/comments';
		}

		// Fichiers récents.
		if (Config::$params['items_recent'])
		{
			$request[] = '{category}/{id}/recent-images';
			$request[] = '{category}/{id}/recent-items';
			$request[] = '{category}/{id}/recent-videos';
		}

		// Moteur de recherche.
		if (Config::$params['search'])
		{
			$request[] = '{category}/{id}/search/{search}';
		}

		// Tags.
		if (Config::$params['tags'])
		{
			$request[] = '{category}/{id}/tag/{id}';
		}

		// Utilisateurs.
		if (Config::$params['users'])
		{
			$request[] = '{category}/{id}/user-favorites/{id}';
			$request[] = '{category}/{id}/user-images/{id}';
			$request[] = '{category}/{id}/user-items/{id}';
			$request[] = '{category}/{id}/user-videos/{id}';
			$request[] = '{category}/{id}/favorites';
		}

		// Votes.
		if (Config::$params['votes'])
		{
			$request[] = '{category}/{id}/votes';
		}

		// Analyse de la requête.
		App::request($request);

		// Paramètres GET par défaut.
		if (!isset($_GET['section']))
		{
			self::_forbidden();
		}
		if (isset($_GET['album_id']))
		{
			$_GET['category_id'] = $_GET['album_id'];
		}
		if (!isset($_GET['category_id']))
		{
			$_GET['category_id'] = 1;
		}
	}

	/**
	 * Clause LIMIT pour la récupération de fichiers
	 * autour du fichier actuel.
	 *
	 * @param int $position
	 * @param int $max
	 *
	 * @return string
	 */
	private static function _sqlLimit(int $position, int $max): string
	{
		$start = $position - $max - 1;
		$items_count = ($max * 2) + 1;

		if ($start < 0)
		{
			$items_count -= -$start;
			$start = 0;
		}

		return "$start,$items_count";
	}
}
?>