<?php
declare(strict_types = 1);

require_once(__DIR__ . '/classes/GalleryUsers.class.php');

// Permission "Ajout de fichiers".
if (!Auth::$groupPerms['upload'])
{
	Gallery::notFound();
	return;
}

Template::set('template',
[
	'content' => 'user_upload.tpl.php',
	'file' => 'user.tpl.php'
]);

$text_limits = sprintf(
	__('Vos fichiers ne doivent pas dépasser %s Mo.'),
	Config::$params['upload_maxfilesize']
);
$text_limits_images = sprintf(
	__('Les images doivent être au format JPEG, GIF, PNG ou WEBP uniquement,'
	. ' et faire %s pixels de coté maximum.'),
	Config::$params['upload_maxwidth'] . ' x ' . Config::$params['upload_maxheight']
);
$text_limits_videos = __('Les vidéos doivent être au format MP4 ou WEBM.');
switch (Auth::$groupPerms['upload_type'])
{
	case 'images' :
		$fileexts = Image::EXT_ARRAY;
		$filetypes = Item::IMAGE_TYPES;
		$types_text = __('images');
		$text_limits .= ' ' . $text_limits_images;
		break;

	case 'videos' :
		$fileexts = Video::EXT_ARRAY;
		$filetypes = Item::VIDEO_TYPES;
		$types_text = __('vidéos');
		$text_limits .= ' ' . $text_limits_videos;
		break;

	default :
		$fileexts = array_merge_recursive(Image::EXT_ARRAY, Video::EXT_ARRAY);
		$filetypes = array_merge_recursive(Item::IMAGE_TYPES, Item::VIDEO_TYPES);
		$types_text = __('images et vidéos');
		$text_limits .= ' ' . $text_limits_videos . ' ' . $text_limits_images;
}
Template::set('upload',
[
	'fileexts' => $fileexts,
	'filetypes' => array_unique(array_map(['Item', 'getTypeMime'], $filetypes)),
	'temp_dir' => Utility::keyHMAC(),
	'text_limits' => $text_limits,
	'text_types' => $types_text,
	'validation' => (int) !Auth::$groupPerms['upload_mode']
]);

$r = GalleryUsers::upload();
if ($r < 0)
{
	Gallery::error();
	return;
}
if ($r < 1)
{
	Gallery::notFound();
	return;
}

$r = GalleryUsers::getInfos();
if ($r < 0)
{
	Gallery::error();
	return;
}
if ($r < 1)
{
	Gallery::notFound();
	return;
}

GalleryUsers::getAlbumsList();
?>