'use strict';

/**
 * Gestion du diaporama.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
function Diaporama(q, p, o)
{
	// Indique si la lecture automatique est en cours.
	var _autoActive = false;

	// Indique si le bouton de la souris est enfoncé
	// pour la modification de la durée d'affichage
	// des images en lecture automatique.
	var _autoDurationMouseDown = false;

	// Indique si un message du mode automatique est affiché.
	var _autoMsgActive = false;

	// Timer pour l'affichage des textes du mode automatique.
	var _autoMsgTimer;

	// Timer pour le changement d'images en lecture automatique.
	var _autoTimer;

	// Indique si une animation du carrousel est en cours.
	var _carouselAnimation = false;

	// Numéro de la 'page' courante du carrousel.
	var _carouselCurrentPage = 0;

	// Position courante dans le carrousel.
	var _carouselCurrentPosition = 1;

	// Nombre maximal de vignettes à afficher dans le carrousel.
	var _carouselMaxThumbs = 0;

	// Plus grande position des images du carrousel.
	var _carouselMaxPosition = 0;

	// Nombre de pages du carrousel.
	var _carouselPagesCount = 0;

	// Informations utiles des fichiers du carrousel.
	var _carouselThumbs;

	// Indique si une animation sur les barres de contrôle est en cours.
	var _controlBarsAnimation = false;

	// Le pointeur de la souris se trouve-t-il sur une barre de contrôle ?
	var _controlBarsOver = false;

	// Position du fichier courant dans la série.
	var _currentPosition = 1;

	// Indique si le bouton de la souris est enfoncé
	// pour le déplacemnt d'une image.
	var _dragMouseDown = false;

	// Contrôle au clavier actif ?
	var _keyboardActive = true;

	// Verrou utilisé lorsqu'un effet de transition entre fichiers est en cours.
	var _itemTransitionLock = false;

	// Informations des fichiers.
	var _items = {};

	// Nombre de fichiers dans la série.
	var _itemsCount = 0;

	// Options.
	var _options =
	{
		// Activer les animations ?
		'animate': true,

		// Durée d'affichage par défaut d'un fichier en lecture automatique.
		// En secondes.
		'autoDuration': 3.0,

		// Durée d'affichage minimale des fichiers en lecture automatique.
		// En secondes.
		'autoDurationMin': 1,

		// Durée d'affichage maximale des fichiers en lecture automatique.
		// En secondes.
		'autoDurationMax': 60,

		// Précision pour la durée d'affichage des fichiers en lecture automatique.
		// En secondes.
		'autoDurationPrecision': 0.5,

		// En lecture automatique, recommencer la lecture depuis
		// le premier fichier lorsqu'on est arrivé au dernier ?
		'autoLoop': false,

		// Démarrer la lecture automatique au lancement du diaporama ?
		'autoStart': false,

		// Afficher par défaut le carrousel ?
		'carousel': true,

		// Épaisseur de la bordure du haut du carrousel.
		'carouselBorderTop': 2, // utile ?

		// Durée de l'effet de transition entre les pages du carrousel.
		'carouselNavDuration': 600,

		// Marge externe des vignettes du carrousel, en pixels.
		'carouselThumbsMargin': 8,

		// Épaisseur de la bordure des vignettes du carrousel, en pixels.
		'carouselThumbsBorder': 1,

		// Dimensions des vignettes du carrousel, en pixels.
		// Valeur minimale : 50.
		'carouselThumbsSize': 80,

		// Lancer le diaporama en mode plein écran ?
		'fullScreen': false,

		// Lancer le diaporama en mode plein écran pour mobile ?
		'fullScreenMobile': true,

		// Cacher les barres de contrôle ?
		'hideControlBars': false,

		// Durée de l'effet de redimensionnement de l'image
		// pour le switch taille réelle / taille redimensionnée.
		// 0 pour désactiver l'animation.
		// En millisecondes.
		'imageResizeDuration': 250,

		// Autoriser la navigation au clavier ?
		'keyboard': true,

		// Largeur maximale jusqu'où l'on considère
		// un écran comme un écran mobile.
		'mobileMaxWidth': 599,

		// La sidebar doit-elle ne pas masquer l'image ?
		'sidebarImageResize': true,

		// Durée de l'effet d'apparition des sidebars.
		// 0 pour désactiver l'animation.
		// En millisecondes.
		'sidebarShowDuration': 250,

		// Durée de l'effet de transition entre images par défaut.
		// En millisecondes.
		'transitionDuration': 500,

		// Effet de transition entre images par défaut.
		'transitionEffect': 'fade'
	};

	// Valeur CSS "overflow" pour <html> et <body>.
	var _overflow = {};

	// Précédentes dimensions du diaporama
	// (avant un changement de dimensions de la zone d'affichage).
	var _previousDiaporamaSize;

	// L'image courante est-elle en taille réelle ?
	var _realsize = false;

	// Verrou utilisé lorsqu'une sidebar est en mouvement.
	var _sidebarMoveLock = false;



	/**
	 * Démarre le diaporama.
	 *
	 * @return void
	 */
	this.start = function()
	{
		// On supprime les barres de défilement du navigateur
		// et on désactive l'affichage de la galerie.
		_overflow =
		{
			html: $('html').css('overflow'),
			body: $('body').css('overflow')
		};
		$('html,body').css('overflow', 'hidden');
		$('#gallery').hide();

		// Si le diaporama a déjà été chargé.
		if (diaporama_init)
		{
			if (_currentPosition != p)
			{
				_deleteImages(true);
				_currentPosition = p;
				_itemsCount = 0;
				_realsize = false;
				_getData(true, false, p, p);
				$('#diaporama_navigation_switch').removeClass('fullsize');
			}

			// Mode plein écran.
			_fullScreen();

			// On affiche le diaporama.
			$('#diaporama').show();

			_carouselChangeSizePosition();

			_changeLoadingSizePosition();

			_keyboardActive = true;

			// Dimensions et position de l'image.
			_changeItemSizePosition(_currentPosition, false, true);

			return;
		}

		// Options.
		_setOptions();

		// Mode plein écran.
		_fullScreen();

		// Redimensionnement du diaporama lors
		// du redimensionnement de la fenêtre du navigateur.
		$(window).resize(function()
		{
			_resize();
		});

		// Événements la lecture automatique.
		_setAutoEvents();

		// Événements sur les boutons.
		_setButtonsEvents();

		// Événements concernant le carrousel.
		_setCarouselEvents();

		// Gestion des favoris.
		_setFavoriteEvents();

		// Gestion des sidebars.
		_setSidebarsEvents();

		// Gestion du swipe.
		_setMobileEvents();

		// Position.
		_currentPosition = p;

		// Récupération des données.
		_getData(true, false, p, p);

		// Durée d'affichage des images en lecture automatique.
		_autoAddDuration();

		// On affiche le diaporama.
		$('#diaporama').show();

		// Gestion de l'affichage des barres de contrôle.
		_setControlBars();

		// Change les dimensions et la position du carrousel.
		_carouselChangeSizePosition();

		// Change la position de l'icône de chargement.
		_changeLoadingSizePosition();

		// Dimensions actuelles du diaporama.
		_previousDiaporamaSize = _getDiaporamaSize();

		// Contrôle au clavier.
		_keyboard();

		// On indique que le diaporama a été initialisé.
		diaporama_init = true;
	};



	/**
	 * Génère une alerte pour afficher un message d'erreur.
	 *
	 * @param string message
	 *
	 * @return void
	 */
	function _alertError(message)
	{
		if (message == '')
		{
			message = 'Unknown error.';
		}
		else if (message.length > 500)
		{
			message = message.slice(0, 500) + '...';
		}
		alert('Error: ' + message);
	}

	/**
	 * Ajoute au diaporama la durée d'affichage
	 * des images lors de la lecture automatique.
	 *
	 * @return void
	 */
	function _autoAddDuration()
	{
		var duration = _options.autoDuration.toString();

		if (!duration.match(/\./))
		{
			duration += '.0';
		}
		$('#diaporama_auto_duration').text(duration + ' s');

		if (diaporama_init && !$('#diaporama_bottom').is(':visible'))
		{
			_autoChangeMessage(duration);
		}
	};

	/**
	 * Change la durée d'affichage des images en lecture automatique.
	 *
	 * @param float i
	 *   Intervalle entre chaque palier de durée.
	 * @param integer d
	 *   Durée entre chaque palier de durée
	 *   lorsqu'on reste appuyé sur le bouton.
	 *
	 * @return void
	 */
	function _autoChangeDuration(i, d)
	{
		// Si le bouton n'est plus appuyé, on ne va pas plus loin.
		if (!_autoDurationMouseDown)
		{
			return;
		}

		// Accélération.
		d = (!d) ? 170 : (d > 20) ? d - 5 : d;

		// La durée doit-être comprise entre une valeur minimale et une valeur maximale.
		if (_options.autoDuration + i >= _options.autoDurationMin
		 && _options.autoDuration + i <= _options.autoDurationMax)
		{
			_options.autoDuration = Math.round((_options.autoDuration + i) * 10) / 10;

			_autoAddDuration();

			setTimeout(function(){ _autoChangeDuration(i, d); }, d);
		}
	};

	/**
	 * Gère le changement d'image de la lecture automatique.
	 *
	 * @param bool start
	 *   Démarre-t-on la lecture automatique ?
	 *
	 * @return void
	 */
	function _autoChangeImage(start)
	{
		// Si la lecture automatique a été stoppée, on casse la boucle.
		if (!_autoActive)
		{
			return;
		}

		if (!start)
		{
			// Tant que l'on est pas à la dernière image, on continue.
			if (_currentPosition != _itemsCount)
			{
				_changeItem('next');
			}

			// Si l'on est arrivé à la dernière image, on recommence
			// à la première image si l'option pour boucler est à true.
			else if (_options.autoLoop)
			{
				_changeItem('first');
			}
		}

		// On attend le chargement de l'image avant de passer à la suivante.
		var img = '#diaporama_item_' + _currentPosition;
		var change = function()
		{
			$('#diaporama_loading').hide();

			var transition_duration = (_options.transitionEffect != 'none' && !start)
				? parseInt(_options.transitionDuration)
				: 0;
			_autoTimer = setTimeout(
				function(){ _autoChangeImage(); },
				(_options.autoDuration * 1000) + transition_duration
			);
		};
		$('#diaporama_loading').show();
		$(img)
			.one('load', function()
			{
				change();
			})
			.each(function()
			{
				if (this.complete || this.readyState == 4 || this.readyState == 'complete')
				{
					$(this).trigger('load');
				}
			});
	}

	/**
	 * Gestion de l'affichage des messages utilisés
	 * pour la lecture automatique.
	 *
	 * @param string text
	 *   Texte à afficher.
	 *
	 * @return void
	 */
	function _autoChangeMessage(text)
	{
		$('#diaporama_auto_message').text(text);

		if (_autoMsgActive)
		{
			$('#diaporama_auto_message').stop(true, true);
			clearTimeout(_autoMsgTimer);
			_autoMsgTimer = undefined;
		}

		_autoMsgActive = true;

		$('#diaporama_auto_message')
			.css('visibility', 'visible')
			.fadeIn(1000, function()
			{
				_autoMsgTimer = setTimeout(
					function()
					{
						$('#diaporama_auto_message')
							.fadeOut(1000, function() { _autoMsgActive = false; });
					},
					2000
				);
			});
	}

	/**
	 * Gestion des pages du carrousel.
	 *
	 * @return void
	 */
	function _carouselChangePages()
	{
		// Détermine le nombre de pages ainsi que la page courante.
		var i = Math.floor(_carouselMaxThumbs) - 1;
		_carouselPagesCount = Math.ceil((_itemsCount - 1) / i);
		_carouselCurrentPage = Math.ceil(_carouselCurrentPosition / i);
		if (_carouselCurrentPage > _carouselPagesCount)
		{
			_carouselCurrentPage = _carouselPagesCount
		}

		// Bouton précédent.
		if (_carouselCurrentPage > 1)
		{
			$('a[data-name="carousel_prev"]').removeClass('disabled');
		}
		else
		{
			$('a[data-name="carousel_prev"]').addClass('disabled');
		}

		// Bouton suivant.
		if (_carouselCurrentPage < _carouselPagesCount)
		{
			$('a[data-name="carousel_next"]').removeClass('disabled');
		}
		else
		{
			$('a[data-name="carousel_next"]').addClass('disabled');
		}
	}

	/**
	 * Change l'indicateur du fichier courant du caroussel.
	 *
	 * @return void
	 */
	function _carouselChangeCurrent()
	{
		if (!_options.carousel)
		{
			return;
		}

		$('#diaporama_carousel_thumbs dl').removeClass('current');
		$('#diaporama_carousel_image_' + _currentPosition).addClass('current');
	}

	/**
	 * Change les dimensions et la position du carrousel.
	 *
	 * @param int width
	 *
	 * @return void
	 */
	function _carouselChangeSizePosition(width)
	{
		var s = _getDiaporamaSize();
		s.availableWidth += (width) ? width : 0;
		var next_icon = $('a[data-name="carousel_next"]');
		var prev_icon = $('a[data-name="carousel_prev"]');
		var next_css =
		{
			'left': s.availableWidth - prev_icon.outerWidth() + 'px'
		};

		var change = function()
		{
			// Détermine le nombre maximum de vignettes que peut contenir le carrousel.
			_carouselMaxThumbs = (s.availableWidth
				- (prev_icon.outerWidth() * 2)
				- _options.carouselThumbsMargin)
				/ (_options.carouselThumbsSize + (_options.carouselThumbsBorder * 2)
				+ _options.carouselThumbsMargin);

			// On regénère les images.
			_carouselMaxPosition = 0;
			_carouselChangeThumbs();
		};

		// Hauteur du carrousel.
		var height_add = (_options.carouselThumbsMargin * 2)
			+ (_options.carouselThumbsBorder * 2) + _options.carouselBorderTop;
		$('#diaporama_carousel').css(
		{
			'height': (_options.carouselThumbsSize + height_add) + 'px'
		});

		// On crée une animation seulement lorsqu'une sidebar est affichée ou cachée.
		if (width && _options.animate)
		{
			next_icon.animate(next_css, _options.sidebarShowDuration, function(){change();});
		}
		else
		{
			next_icon.css(next_css);
			change();
		}
	}

	/**
	 * Change les vignettes du carrousel.
	 *
	 * @param string nav
	 *
	 * @return void
	 */
	function _carouselChangeThumbs(nav)
	{
		if (_carouselThumbs === undefined)
		{
			return;
		}

		// Retourne la position à partir de laquelle on
		// doit afficher les vignettes dans le carrousel.
		var get_carousel_start = function()
		{
			var i = Math.floor(_carouselMaxThumbs) - 1;

			return ((Math.ceil(_carouselCurrentPosition / i) - 1) * i) + 1;
		};

		if (_currentPosition >= _carouselMaxPosition)
		{
			var center, width, height, html = '', n = 0;
			var thumbs_count = Math.floor(_carouselMaxThumbs) - 1;
			var max_thumbs = Math.ceil(_carouselMaxThumbs);

			// Position à partir de laquelle on affiche les vignettes.
			var start = get_carousel_start();
			if (start == _itemsCount)
			{
				start -= thumbs_count;
				if (start < 1)
				{
					start = 1;
				}
			}

			// Dans le cas d'une navigation dans le carrousel,
			// on supprimera les vignettes après l'animation.
			if (nav)
			{
				_carouselAnimation = true;
				_carouselCurrentPosition = start;

				var s = _getDiaporamaSize();
				var margin_left = thumbs_count * (_options.carouselThumbsSize
					+ (_options.carouselThumbsBorder * 2) + _options.carouselThumbsMargin);

				if (nav == 'prev')
				{
					$('#diaporama_carousel_thumbs dl:gt('
						+ (max_thumbs - thumbs_count) + ')').addClass('old');
				}
				else
				{
					$('#diaporama_carousel_thumbs dl:lt('
						+ thumbs_count + ')').addClass('old');
				}
			}

			// Sinon, on supprime tout de suite toutes les vignettes,
			// mais seulement si c'est nécessaire.
			else
			{
				var first_position = ($('#diaporama_carousel_thumbs dl').length)
					? parseInt($('#diaporama_carousel_thumbs dl:first').attr('id')
						.replace(/diaporama_carousel_image_(\d+)/, '$1'))
					: start;
				if (first_position != start)
				{
					$('#diaporama_carousel_thumbs').empty();
				}
			}

			// Création et insertion des nouvelles vignettes.
			for (var pos in _carouselThumbs)
			{
				if (pos < start || $('#diaporama_carousel_image_' + pos).is('dl'))
				{
					continue;
				}
				if (nav == 'prev' && n == thumbs_count)
				{
					break;
				}
				n++;

				var duration = _carouselThumbs[pos].is_video
					? '<span class="duration"><span>&#xea1c;</span> '
						+ _carouselThumbs[pos].duration + '</span>'
					: '';

				width = _options.carouselThumbsSize;
				height = _options.carouselThumbsSize;
				html += '<dl id="diaporama_carousel_image_' + pos + '"><dt><a rel="' + n
					+ '" style="width:' + width + 'px;height:' + height + 'px;">' + duration
					+ '<img src="' + _carouselThumbs[pos].source + '"></a></dt></dl>' + "\n";

				_carouselMaxPosition = pos;

				if (n == max_thumbs)
				{
					break;
				}
			}
			if (nav == 'prev')
			{
				$('#diaporama_carousel_thumbs')
					.css({'margin-left': '-' + margin_left + 'px'})
					.prepend(html);
			}
			else if (html !== '')
			{
				$('#diaporama_carousel_thumbs').append(html);
			}

			// Suppression des anciennes vignettes lors
			// de la navigation dans le carrousel.
			if (nav)
			{
				var nav_css = (nav == 'prev')
					? {'margin-left': 0}
					: {'margin-left': '-' + margin_left + 'px'};

				$('#diaporama_carousel_thumbs').animate(
					nav_css,
					_options.animate ? _options.carouselNavDuration : 0,
					'swing',
					function()
					{
						$('#diaporama_carousel_thumbs dl.old').remove();
						$('#diaporama_carousel_thumbs').css('margin-left', 0);

						_getData(false, false, 0, _carouselCurrentPosition);
						_carouselAnimation = false;
					}
				);
			}

			// Gestion de l'événement "click" sur les vignettes du carrousel.
			$('#diaporama_carousel_thumbs dl').click(function()
			{
				var new_current_position = parseInt($(this).attr('id')
					.replace(/diaporama_carousel_image_(\d+)/, '$1'));

				if (new_current_position == _currentPosition)
				{
					return;
				}

				// On réinitialise le diaporama et on récupère les informations des images.
				_options.autoStart = false;
				_carouselReload(new_current_position);

				// Position du fichier courant dans le carrousel.
				_carouselChangeCurrent();

				// Si la vignette cliquée est la dernière, inutile de changer le carrousel,
				// sauf si cette dernière vignette apparaît tronquée dans le carrousel.
				if (get_carousel_start() == _itemsCount)
				{
					return;
				}

				// On regénère le carrousel.
				if ($(this).find('a').attr('rel') >= Math.floor(_carouselMaxThumbs))
				{
					_carouselMaxPosition = 0;
					_getData(false, false, 0, new_current_position);
				}

				// On réinitialise la lecture automatique.
				if (typeof _autoTimer == 'number')
				{
					if ((new_current_position == _itemsCount) && !_options.autoLoop)
					{
						$('.diaporama_icon[data-name="stop"]').trigger('click');
					}
					else
					{
						clearTimeout(_autoTimer);
						_autoTimer = undefined;
						_autoTimer = setTimeout(
							function(){ _autoChangeImage(); },
							_options.autoDuration * 1000
						);
					}
				}
			});

			// Change les pages du carrousel.
			_carouselChangePages();
		}

		// Position de l'image courante dans le carrousel.
		if (!nav)
		{
			_carouselChangeCurrent();
		}
	}

	/**
	 * Recharge tout le diaporama depuis la position p.
	 *
	 * @param int pos
	 *
	 * @return void
	 */
	function _carouselReload(pos)
	{
		_currentPosition = pos;
		_itemsCount = 0;
		_realsize = false;
		_changeButtonsNavigation();
		_deleteImages(true);
		_getData(true, false, pos, pos);
	}

	/**
	 * Modifie les boutons de navigation.
	 *
	 * @return void
	 */
	function _changeButtonsNavigation()
	{
		var buttons =
		{
			'first': 1,
			'prev': (_currentPosition <= 2) ? 1 : _currentPosition - 1,
			'next': (_currentPosition >= _itemsCount) ? _itemsCount : _currentPosition + 1,
			'last': _itemsCount
		};

		for (var button in buttons)
		{
			if (_items[buttons[button]] && _currentPosition != buttons[button])
			{
				$('a[data-name="' + button + '"]').removeClass('disabled');
			}
			else
			{
				$('a[data-name="' + button + '"]').addClass('disabled');
			}
		}
	}

	/**
	 * Change le bouton du switch taille réelle / taille redimensionnée de l'image.
	 *
	 * @return void
	 */
	function _changeButtonSwitch()
	{
		if (_items[_currentPosition] === undefined)
		{
			return;
		}

		// Récupération des dimensions du diaporama.
		var s = _getDiaporamaSize();
		var item = $('#diaporama_item_' + _currentPosition);
		item.unbind('mousedown');

		// Si c'est une vidéo, on empêche le switch en taille réelle.
		if (_items[_currentPosition].is_video)
		{
			$('#diaporama_switch').removeClass('fullsize').addClass('disabled');
			return;
		}

		// Si l'image est plus grande que la zone d'affichage, on met
		// en place le bouton adéquat.
		if (_items[_currentPosition].width_resized > s.availableWidth
		 || _items[_currentPosition].height_resized > s.availableHeight)
		{
			$('#diaporama_switch').removeClass('fullsize disabled');

			if (_realsize)
			{
				$('#diaporama_switch').addClass('fullsize');
				$('#diaporama_item_' + _currentPosition).css('cursor', 'move');
			}
			else
			{
				$('#diaporama_item_' + _currentPosition).css('cursor', 'default');
			}
		}

		// Si l'image peut tenir dans la zone d'affichage, on désactive le bouton.
		else
		{
			$('#diaporama_switch').removeClass('fullsize').addClass('disabled');
		}

		// Déplacement de l'image.
		if (_realsize)
		{
			item.mousedown(function()
			{
				_dragMouseDown = true;
				return _dragImage(this, event);
			});

			// Version mobile.
			document.getElementById('diaporama_item_' + _currentPosition)
			.addEventListener('touchstart', function(e)
			{
				_dragImageTouch(e);
			});
		}
	}

	/**
	 * Change les dimensions et la position (coordonnées spatiales) de l'image
	 * en fonction de l'espace disponible dans la zone d'affichage.
	 *
	 * @param number position
	 *   Position du fichier dans la série (entre 1 et itemsCount).
	 *   A ne pas confondre avec la position (top, left) de l'image
	 *   dans la zone d'affichage.
	 * @param bool animate
	 *   Doit-on autoriser l'animation de l'image ?
	 * @param bool visible
	 *   Doit-on afficher l'image ?
	 * @param bool r
	 *   Doit-on retourner les règles CSS au lieu de les appliquer à l'image ?
	 * @param number width
	 *   Largeur à ajouter à la largeur de la zone d'affichage.
	 *
	 * @return mixed
	 *   Retourne les valeurs CSS, si demandées.
	 */
	function _changeItemSizePosition(position, animate, visible, return_css, add_width)
	{
		if (_items[position] === undefined
		|| !$('#diaporama_item_' + position).length)
		{
			return;
		}

		// Récupération des dimensions du diaporama.
		var s = _getDiaporamaSize();
		s.availableWidth += (add_width) ? add_width : 0;

		// Dimensions de l'image.
		var img_width = _items[position].width_resized;
		var img_height = _items[position].height_resized;
		var img_width_resize = img_width;
		var img_height_resize = img_height;
		var width_ratio = img_width / s.availableWidth;
		var height_ratio = img_height / s.availableHeight;

		if (!_realsize)
		{
			if ((img_width > s.availableWidth) && (width_ratio >= height_ratio))
			{
				img_width_resize = s.availableWidth;
				img_height_resize = img_height / width_ratio;
			}
			if ((img_height > s.availableHeight) && (height_ratio >= width_ratio))
			{
				img_width_resize = img_width / height_ratio;
				img_height_resize = s.availableHeight;
			}
		}

		// Position de l'image.
		var img_offset = $('#diaporama_item_' + position).offset();
		var img_current_offset = $('#diaporama_item_' + position).offset();

		// Position de l'image, largeur : modification de la taille de l'image.
		if (img_width_resize <= s.availableWidth
		|| img_width_resize != $('#diaporama_item_' + position).width())
		{
			img_offset.left = (s.availableWidth - img_width_resize) / 2;
		}

		// Position de l'image, largeur : modification de la taille de la zone d'affichage.
		else if (img_width_resize > s.availableWidth
		&& s.availableWidth != _previousDiaporamaSize.availableWidth)
		{
			img_offset.left += (s.availableWidth - _previousDiaporamaSize.availableWidth) / 2;

			var right = img_width_resize - s.availableWidth + img_offset.left;
			if (right < 0)
			{
				img_offset.left -= right;
			}
			if (img_offset.left > 0)
			{
				img_offset.left = 0;
			}

			img_offset.left = img_offset.left;
		}

		// Position de l'image, hauteur : modification de la taille de l'image.
		if (img_height_resize <= s.availableHeight
		|| img_height_resize != $('#diaporama_item_' + position).height())
		{
			img_offset.top = ((s.availableHeight - img_height_resize) / 2) + s.barTopHeight;
		}

		// Position de l'image, hauteur : modification de la taille de la zone d'affichage.
		else if (img_height_resize > s.availableHeight
		&& s.availableHeight != _previousDiaporamaSize.availableHeight)
		{
			img_offset.top += ((s.availableHeight - _previousDiaporamaSize.availableHeight) / 2);
			var bottom = img_height_resize
				- (s.availableHeight + s.barBottomHeight + s.carouselHeight)
				+ img_offset.top;
			if (bottom < 0)
			{
				img_offset.top -= bottom;
			}
			if (img_offset.top > s.barTopHeight)
			{
				img_offset.top = s.barTopHeight;
			}

			img_offset.top = img_offset.top;
		}

		// Valeurs CSS.
		var css =
		{
			top: Math.round(img_offset.top) + 'px',
			left: Math.round(img_offset.left) + 'px',
			width: Math.round(img_width_resize) + 'px',
			height: Math.round(img_height_resize) + 'px'
		};

		// On applique les nouvelles propriétés à l'image.
		if (!return_css)
		{
			if (animate && $('#diaporama_item_' + position).is(':visible')
			&& _options.imageResizeDuration > 0
			&& _options.animate)
			{
				$('#diaporama_item_' + position).animate(css, _options.imageResizeDuration);
			}
			else
			{
				$('#diaporama_item_' + position).css(css);
			}
			$('#diaporama_favorite').css({left: s.availableWidth + 'px'});
		}

		// Doit-on afficher l'image ?
		if (visible)
		{
			$('#diaporama_item_' + position).show();
		}

		// Doit-on retourner les valeurs CSS ?
		if (return_css)
		{
			return css;
		}

		// Pour le fichier courant.
		if (position == _currentPosition)
		{
			_changeButtonSwitch();
			_changePosition();
		}
	}

	/**
	 * Change l'image en fonction du bouton de navigation cliqué.
	 *
	 * @param object button
	 *   Bouton de navigation qui a été cliqué.
	 * @param boolean click
	 *   Indique si la fonction a été appelée par un click.
	 *
	 * @return void
	 */
	function _changeItem(button, click)
	{
		// On met sur pause toutes les vidéos.
		$('#diaporama_item video').each(function()
		{
			this.pause();
		});

		// On détermine la position du fichier à afficher.
		var new_current_position;
		switch (button)
		{
			case 'first' :
				new_current_position = 1;
				break;

			case 'prev' :
				new_current_position = _currentPosition - 1;
				break;

			case 'next' :
				new_current_position = _currentPosition + 1;
				break;

			case 'last' :
				new_current_position = _itemsCount;
				break;
		}

		// Si l'image n'existe pas
		// ou correspondant à une valeur impossible
		// on bien si un effet de transition entre images est en cours,
		// on ne va pas plus loin.
		if (!$('#diaporama_item_' + new_current_position).length
		|| new_current_position < 1 || new_current_position > _itemsCount
		|| _itemTransitionLock || _sidebarMoveLock)
		{
			return;
		}

		// On redimensionne l'image à afficher.
		_realsize = false;
		$('#diaporama_navigation_switch').removeClass('fullsize');

		// On indique les bonnes dimensions et position à l'image que l'on va afficher.
		_changeItemSizePosition(new_current_position);

		// On modifie la valeur de la position courante.
		var old_current_position = _currentPosition;
		_currentPosition = new_current_position;

		// Transition entre l'image courante et l'image demandée.
		_itemTransition(old_current_position, new_current_position, button);

		// On effectue une nouvelle requête ajax, mais seulement
		// après l'effet de transition si un effet de transition est activé,
		// afin d'éviter que l'effet de transition ne soit saccadé.
		if (_options.transitionEffect == 'none')
		{
			_getData(false, false, _currentPosition, _currentPosition);
			_changeItemInfos();
		}
		else
		{
			setTimeout(function()
			{
				_getData(false, false, _currentPosition, _currentPosition);
				_changeItemInfos();
			},
			_options.transitionDuration);
		}

		// Change les boutons de navigation.
		_changeButtonsNavigation();

		// Change le bouton de switch.
		_changeButtonSwitch();

		// Change la position.
		_changePosition();

		// En lecture automatique, si l'on est arrivé à la dernière
		// image et que l'option pour boucler est désactivée,
		// on stoppe la lecture automatique.
		if (_autoActive && new_current_position == _itemsCount && !_options.autoLoop)
		{
			$('.diaporama_icon[data-name="stop"]').click();
		}

		// Si le changement d'image s'est fait manuellement,
		// et que la lecture automatique est en cours,
		// alors on réinitialise la lecture automatique.
		if (click && typeof _autoTimer == 'number')
		{
			// On stoppe la lecture automatique.
			clearTimeout(_autoTimer);
			_autoTimer = undefined;

			// On redémarre la lecture automatique.
			_autoChangeImage(true);
		}
	}

	/**
	 * Change les informations du fichier courant
	 * affichées dans le diaporama.
	 *
	 * @return void
	 */
	function _changeItemInfos()
	{
		if (_items[_currentPosition] === undefined)
		{
			return;
		}

		var current = _items[_currentPosition];
		var html;

		// Identifiant du fichier.
		diaporama_item_id = current.id;

		// Fil d'Ariane.
		$('#diaporama_breadcrumb').html(current.breadcrumb);

		// Bouton de téléchargement.
		if (current.download && $('#small_screen').is(':hidden'))
		{
			$('.diaporama_icon[data-name="download"]')
				.attr('title', diaporama_download_text
					+ ' (' + current.stats.filesize.formated + ')')
				.attr('href', $('<input>').html(current.download).text())
				.show();
			$('#diaporama_item_' + _currentPosition).unbind('contextmenu');
		}
		else
		{
			$('.diaporama_icon[data-name="download"]').hide();

			// Anti clic droit.
			$('#diaporama_item_' + _currentPosition).bind('contextmenu', function()
			{
				return false;
			});
		}

		// Édition.
		if (current.user.edit && $('#small_screen').is(':hidden'))
		{
			$('.diaporama_icon[data-name="edit"]').show();

			$('#diaporama_edit_title').val($('<input>').html(current.title).text());
			$('#diaporama_edit_filename').val($('<input>').html(current.filename).text());
			$('#diaporama_edit_desc').val($('<input>').html(current.description).text());

			var tags = '';
			if (current.tags.length)
			{
				for (var tag in current.tags)
				{
					tags += ', ' + current.tags[tag].name;
				}
				tags = tags.substring(2);
			}
			$('#diaporama_edit_tags').val($('<input>').html(tags).text());
		}
		else
		{
			if ($('.diaporama_icon[data-name="edit"]').hasClass('active'))
			{
				$('.diaporama_icon[data-name="edit"]').click();
			}
			$('.diaporama_icon[data-name="edit"]').hide();
		}

		// Informations : description.
		if (current.description)
		{
			$('#diaporama_sidebar_description p').html(current.description);
			$('#diaporama_sidebar_description').show();
		}
		else
		{
			$('#diaporama_sidebar_description').hide();
		}

		// Informations : statistiques.
		for (var stat in current.stats)
		{
			if (Object.keys(current.stats[stat]).length)
			{
				if (stat == 'rating')
				{
					html = '';
					var entity;
					for (var key in current.stats[stat].array)
					{
						if (current.stats[stat].array[key] == 1)
						{
							entity = '&#xe9d9;';
						}
						else if (current.stats[stat].array[key] == 0)
						{
							entity = '&#xe9d7;';
						}
						else
						{
							entity = '&#xe9d8;';
						}
						html += '<span class="rating">' + entity + '</span>';
					}
					$('#diaporama_sidebar_stats_' + stat + ' span')
						.html(html).attr('title', current.stats[stat].formated);
				}
				else
				{
					$('#diaporama_sidebar_stats_' + stat + ' span')
						.text(current.stats[stat].short);
				}
				$('#diaporama_sidebar_stats_' + stat).show();
			}
			else
			{
				$('#diaporama_sidebar_stats_' + stat).hide();
			}
		}

		// Informations : utilisateur.
		if (current.owner)
		{
			$('#diaporama_sidebar_owner .diaporama_sidebar_content').html(current.owner);
		}
		else
		{
			$('#diaporama_sidebar_owner').hide();
		}

		// Informations : propriétés.
		$('#diaporama_sidebar_properties_type span').text(current.type_text);
		$('#diaporama_sidebar_properties_filesize span').text(current.stats.filesize.formated);
		$('#diaporama_sidebar_properties_size span:first-child')
			.text(current.width + ' x ' + current.height);
		if (current.is_video)
		{
			$('#diaporama_sidebar_properties_duration').show();
			$('#diaporama_sidebar_properties_duration span').text(current.duration_text);
		}
		else
		{
			$('#diaporama_sidebar_properties_duration').hide();
		}
		if (current.date_created)
		{
			$('#diaporama_sidebar_properties_date_created').show();
			$('#diaporama_sidebar_properties_date_created span').text(current.date_created_text);
		}
		else
		{
			$('#diaporama_sidebar_properties_date_created').hide();
		}

		// Informations : tags.
		if (current.tags.length)
		{
			html = '<ul>';
			for (var tag in current.tags)
			{
				html += '<li><a href="' + current.tags[tag].link + '">'
					+ current.tags[tag].name + '</a></li>';
			}
			html += '</ul>';
			$('#diaporama_sidebar_tags .diaporama_sidebar_content').html(html);
			$('#diaporama_sidebar_tags').show();
		}
		else
		{
			$('#diaporama_sidebar_tags').hide();
		}

		// Informations : métadonnées.
		if (Object.keys(current.exif).length)
		{
			html = '<ul>';
			for (var data in current.exif)
			{
				html += '<li>' + current.exif[data].name + ' : <span>';
				if (current.exif[data].link)
				{
					html += '<a href="' + current.exif[data].link + '">'
						+ current.exif[data].value + '</a>';
				}
				else
				{
					html += current.exif[data].value;
				}
				html += '</span></li>';
			}
			html += '</ul>';
			$('#diaporama_sidebar_exif').show();
			$('#diaporama_sidebar_exif .diaporama_sidebar_content').html(html);
		}
		else
		{
			$('#diaporama_sidebar_exif').hide();
		}

		// Dans les favoris ?
		var fav = $('.diaporama_icon[data-name="favorite"]');
		if (fav.length)
		{
			if (_items[_currentPosition].user.in_favorites)
			{
				fav.attr('title', diaporama_favorites_remove_text);
				fav.addClass('active');
				$('#diaporama_favorite').addClass('in');
			}
			else
			{
				fav.attr('title', diaporama_favorites_add_text);
				fav.removeClass('active');
				$('#diaporama_favorite').removeClass('in');
			}
		}

		// Carrousel.
		_carouselChangeCurrent();
	}

	/**
	 * Change la taille du conteneur pour l'icône de chargement.
	 *
	 * @param int width
	 *
	 * @return void
	 */
	function _changeLoadingSizePosition(width)
	{
		var s = _getDiaporamaSize();
		s.availableWidth += (width) ? width : 0;
		$('#diaporama_loading').css(
		{
			top: s.barTopHeight,
			width: s.availableWidth,
			height: s.availableHeight
		});
	}

	/**
	 * Change le texte pour la position dans la série.
	 *
	 * @return void
	 */
	function _changePosition()
	{
		if (_items[_currentPosition] !== undefined)
		{
			$('#diaporama_position').text($(window).width() <= _options.mobileMaxWidth
				? _items[_currentPosition].position : _items[_currentPosition].position_text);
		}
	}

	/**
	 * Création de l'élément "<img>" ou "<video>" du fichier correspondant à "position".
	 *
	 * @param string position
	 *   Position du fichier dans la série.
	 *
	 * @return void
	 */
	function _createItem(position)
	{
		if (_items[position] === undefined
		|| $('#diaporama_item_' + position).length)
		{
			return;
		}

		if (_items[position].is_video)
		{
			var loop = diaporama_video_loop ? ' loop' : '';
			var muted = diaporama_video_muted ? ' muted' : '';
			var html =
				'<video' + loop + muted
				+ ' style="display:none" class="diaporama_item" id="diaporama_item_'
				+ position + '" controls onloadstart="this.volume=0.5" poster="'
				+ _items[position].poster + '"><source src="' + _items[position].source
				+ '" type="' + _items[position].type_mime + '"></video>';
			$('#diaporama_item').append(html);
		}
		else
		{
			$('#diaporama_item').append(
				'<img class="diaporama_item" id="diaporama_item_' + position
				+ '" style="display:none" src="' + _items[position].source + '">'
			);
		}

		// Icône de chargement.
		if (position == _currentPosition)
		{
			$('#diaporama_loading').hide();
		}
	}

	/**
	 * Supprime les images inutiles pour ne pas encombrer et ralentir le diaporama.
	 *
	 * @param bool all
	 *   Doit-on supprimer toutes les images ?
	 *
	 * @return void
	 */
	function _deleteImages(all)
	{
		$('.diaporama_item').each(function()
		{
			var position = $(this).attr('id').replace(/diaporama_item_/, '');
			if (_items[position] === undefined || all)
			{
				$(this).remove();
			}
		});
	}

	/**
	 * Fonction de drag basée sur le code posté par
	 * Lasse Reichstein Nielsen dans le groupe comp.lang.javascript
	 * en janvier 2004 et adapté pour iGalerie et jQuery.
	 *
	 * @param object img
	 * @param object evt
	 *
	 * @return bool
	 */
	function _dragImage(img, evt)
	{
		// Récupération des dimensions du diaporama.
		var s = _getDiaporamaSize();

		var x = img.offsetLeft;
		var y = img.offsetTop;
		var mx = evt.pageX;
		var my = evt.pageY;

		$(document).mousemove(function(evt)
		{
			if (!_dragMouseDown)
			{
				return;
			}

			var newmx = evt.pageX;
			var newmy = evt.pageY;
			x += newmx - mx;
			y += newmy - my;
			mx = newmx;
			my = newmy;

			if ($(img).width() > s.availableWidth)
			{
				if (x > 0)
				{
					x = 0;
				}
				else if (x < (s.availableWidth - $(img).width()))
				{
					x = s.availableWidth - $(img).width();
				}
			}
			else
			{
				x = $(img).css('left');
			}

			if ($(img).height() > s.availableHeight)
			{
				if (y > s.barTopHeight)
				{
					y = s.barTopHeight;
				}
				else if (y < $(window).height() - s.barBottomHeight
					- s.carouselHeight - $(img).height())
				{
					y = $(window).height() - s.barBottomHeight
						- s.carouselHeight - $(img).height();
				}
			}
			else
			{
				y = $(img).css('top');
			}

			$(img).css({
				left: x + 'px',
				top: y + 'px'
			});

			return false;
		});

		$(document).mouseup(function()
		{
			_dragMouseDown = false;

			return false;
		});

		return false;
	}

	/**
	 * Fonction de drag pour mobile.
	 *
	 * @param object evt
	 *
	 * @return void
	 */
	function _dragImageTouch(evt)
	{
		var img = document.getElementById('diaporama_item_' + _currentPosition);
		var s = _getDiaporamaSize();

		var x = img.offsetLeft;
		var y = img.offsetTop;

		var mx = evt.changedTouches[0].pageX;
		var my = evt.changedTouches[0].pageY;

		img.addEventListener('touchmove', function(e)
		{
			//e.preventDefault();

			var touchobj = e.changedTouches[0];

			var newmx = touchobj.pageX;
			var newmy = touchobj.pageY;
			x += newmx - mx;
			y += newmy - my;
			mx = newmx;
			my = newmy;

			if (img.width > s.availableWidth)
			{
				if (x > 0)
				{
					x = 0;
				}
				else if (x < (s.availableWidth - img.width))
				{
					x = s.availableWidth - img.width;
				}
			}
			else
			{
				x = parseInt(img.style.left);
			}

			if (img.height > s.availableHeight)
			{
				if (y > s.barTopHeight)
				{
					y = s.barTopHeight;
				}
				else if (y < $(window).height() - s.barBottomHeight
					- s.carouselHeight - img.height)
				{
					y = $(window).height() - s.barBottomHeight
						- s.carouselHeight - img.height;
				}
			}
			else
			{
				y = parseInt(img.style.top);
			}

			img.style.left = x + 'px';
			img.style.top = y + 'px';

		}, false);
	}

	/**
	 * Affichage en mode plein écran.
	 *
	 * @return void
	 */
	function _fullScreen()
	{
		if (_options.fullScreen || (_options.fullScreenMobile && _isMobile()))
		{
			var el = document.body;
			var fs = el.requestFullScreen || el.webkitRequestFullScreen
				 || el.mozRequestFullScreen || el.msRequestFullScreen;
			if (fs)
			{
				fs.call(el);
			}
		}
	}

	/**
	 * Sortie du mode plein écran.
	 *
	 * @return void
	 */
	function _fullScreenExit()
	{
		if (document.fullScreen || document.mozFullScreen
		|| document.webkitIsFullScreen || document.msFullscreenElement)
		{
			if (document.exitFullscreen)
			{
				document.exitFullscreen();
			}
			else if (document.mozCancelFullScreen)
			{
				document.mozCancelFullScreen();
			}
			else if (document.webkitExitFullscreen)
			{
				document.webkitExitFullscreen();
			}
			else if (document.msExitFullscreen)
			{
				document.msExitFullscreen();
			}
		}
	}

	/**
	 * Récupération des données depuis le serveur.
	 *
	 * @param bool start
	 *   Utilise-t-on _getImages() pour la première fois ?
	 * @param bool only_item_infos
	 *   Ne changer que les informations du fichier courant ?
	 * @param int carousel_position
	 *   Position du fichier courant dans la série.
	 *   0 pour ne pas récupérer d'informations du fichier.
	 * @param int carousel_position
	 *   Position dans le carrousel.
	 *   0 pour ne pas récupérer d'informations du carrousel.
	 *
	 * @return void
	 */
	function _getData(start, only_item_infos, item_position, carousel_position)
	{
		if (start)
		{
			$('#diaporama_loading').show();
		}

		_carouselCurrentPosition = carousel_position > 0
			? carousel_position
			: item_position;

		$.post(
			gallery_path + '/ajax.php',
			{
				anticsrf: anticsrf,
				section: 'diaporama',
				q: q,
				get_item: item_position > 0 ? 1 : 0,
				get_carousel: carousel_position > 0 ? 1 : 0,
				item_position: item_position,
				item_preload: 2,
				carousel_position: _carouselCurrentPosition
			},
			function(r)
			{
				if (typeof r != 'object' || r === null)
				{
					return;
				}
				switch (r.status)
				{
					// Erreur.
					case 'error' :
						$('.diaporama_sidebar[data-name="informations"] > div').hide();
						console.log(r.message);
						_alertError(r.message);
						break;

					// Aucun résultat.
					case 'no_result' :
						break;

					// Succès.
					case 'success' :
						$('.diaporama_sidebar[data-name="informations"] > div').show();

						// Gestion du carrousel.
						var carousel = function()
						{
							// Informations des vignettes du carrousel.
							_carouselThumbs = r.carousel;

							// Si le nombre d'images de la section courante
							// n'est plus le même, on recharge le diaporama.
							if (_itemsCount > 0 && r.count != _itemsCount)
							{
								_carouselReload(_currentPosition);
							}

							// Si des images ont été supprimées, on recharge le diaporama.
							if (_carouselThumbs[carousel_position] === undefined && r.count > 0)
							{
								_carouselReload(parseInt(r.count));
								return;
							}

							_itemsCount = r.count;
							_carouselChangeThumbs();
						};
						if (carousel_position)
						{
							carousel();
						}

						// Si on ne doit pas récupérer les informations du fichier courant,
						// on ne va pas plus loin.
						if (!item_position)
						{
							break;
						}

						// Change uniquement les informations
						// du fichier courant ?
						if (only_item_infos)
						{
							_items = r.items;
							_changeItemInfos();
							break;
						}

						// Si la série ne contient plus aucun fichier,
						// on vide l'interface du diaporama.
						if (r.count == 0)
						{
							$('#diaporama_item,#diaporama_breadcrumb,#diaporama_position,'
								+ '.diaporama_sidebar_inner,#diaporama_carousel_thumbs,'
								+ '#diaporama_auto_message,#diaporama_auto_mode').empty();
							$('.diaporama_icon[data-name="download"],'
								+ '.diaporama_icon[data-name="favorite"]')
								.unbind().removeClass('active').removeAttr('title');
							$('a[data-name="carousel_prev"],a[data-name="carousel_next"]')
								.unbind().addClass('disabled');
							$('#diaporama_navigation > a').unbind().addClass('disabled');
							$('#diaporama_auto_mode > a').unbind();
							$('#diaporama_switch').unbind().removeClass().addClass('disabled');
							_carouselThumbs = undefined;
							_currentPosition = 0;
						}

						// Si la série ou le fichier courant ont été modifié,
						// on recrée tous les fichiers.
						var remake = false;
						for (var pos in r.items)
						{
							if (_items[pos] === undefined
							 || _items[pos].md5 == r.items[pos].md5)
							{
								continue;
							}

							if (_currentPosition > r.count)
							{
								_currentPosition = r.count;
							}

							_deleteImages(true);
							remake = true;
							break;
						}

						_items = r.items;
						_itemsCount = r.count;

						// Création des fichiers.
						for (var pos in r.items)
						{
							var current = _currentPosition == pos;

							if ((!start && !remake) && current)
							{
								continue;
							}

							_createItem(pos);
							_changeItemSizePosition(pos, false, current);

							if ((start || remake) && current)
							{
								_changeItemInfos();
							}
						}

						// Boutons de navigation.
						_changeButtonsNavigation();

						// On indique de regénérer le carrousel.
						if (remake)
						{
							$('#diaporama_carousel_thumbs').empty();
						}

						// Suppression des images inutiles.
						_deleteImages();

						// Démarrage de la lecture automatique au lancement ?
						if (start && _options.autoStart)
						{
							$('.diaporama_icon[data-name="start"]').click();
						}

						// On met à jour les informations (notamment pour le nombre de vues).
						_changeItemInfos();

						// Changement des vignettes du carrousel.
						_carouselMaxPosition = 0;
						_carouselChangeThumbs();
						break;

					default :
						console.log(r);
				}
			},
			'json'
		);
	}

	/**
	 * Retourne les dimensions des éléments du diaporama.
	 *
	 * @return object
	 */
	function _getDiaporamaSize()
	{
		var s =
		{
			'barBottomHeight': 0, 'barTopHeight': 0,
			'carouselHeight': 0, 'sidebarWidth': 0
		};

		// Largeur de la sidebar.
		if (_options.sidebarImageResize && $('.diaporama_sidebar').is(':visible'))
		{
			s.sidebarWidth = $('.diaporama_sidebar').outerWidth();
		}

		// Hauteur du carrousel.
		if (_options.carousel && !_options.hideControlBars)
		{
			s.carouselHeight = $('#diaporama_carousel').outerHeight();
		}

		// Hauteur des barres du haut et du bas.
		if (!_options.hideControlBars)
		{
			s.barTopHeight = $('#diaporama_top').outerHeight();
			s.barBottomHeight = $('#diaporama_bottom').outerHeight();
		}

		// Dimensions de la zone d'affichage.
		s.availableHeight = $(window).height()
			- s.barTopHeight - s.barBottomHeight - s.carouselHeight;
		s.availableWidth = $(window).width() - s.sidebarWidth;

		return s;
	}

	/**
	 * Détermine si l'on est sur un mobile.
	 *
	 * @return bool
	 */
	function _isMobile()
	{
		var touchscreen = !!('ontouchstart' in window)
			|| !!('ontouchstart' in document.documentElement)
			|| !!window.ontouchstart || !!window.Touch || !!window.onmsgesturechange
			|| (window.DocumentTouch && window.document instanceof window.DocumentTouch);

		return touchscreen && $(window).width() <= _options.mobileMaxWidth;
	}

	/**
	 * Effectue la transition entre le fichier courante et le fichier demandé.
	 *
	 * @param int old_position
	 * @param int new_position
	 * @param string button
	 *
	 * @return void
	 */
	function _itemTransition(old_position, new_position, button)
	{
		var s = _getDiaporamaSize();
		var img_old_offset = $('#diaporama_item_' + old_position).offset();
		var duration = parseInt(_options.transitionDuration);
		var transition = _options.transitionEffect;

		_itemTransitionLock = true;

		// Effet aléatoire.
		if (_options.transitionEffect == 'random')
		{
			var nb = $('#diaporama_transitions_effect option').length - 2;
			var rand = Math.floor(Math.random() * nb) + 1;
			transition = document.getElementById('diaporama_transitions_effect')
				.getElementsByTagName('option')[rand].value;
		}

		switch (transition)
		{
			case 'fade' :
				$('#diaporama_item_' + old_position).fadeOut(duration);
				$('#diaporama_item_' + new_position).fadeIn(duration);
				break;

			case 'slideX' :
			case 'slideY' :

				// Fichier actuel.
				var old_css_animate = {};
				var old_css = { display : 'none' };
				if (transition == 'slideX')
				{
					old_css_animate.left = (button == 'next' || button == 'last')
						? (img_old_offset.left - s.availableWidth) + 'px'
						: (img_old_offset.left + s.availableWidth) + 'px';
					old_css.left = img_old_offset.left + 'px';
				}
				else
				{
					old_css_animate.top = (button == 'next' || button == 'last')
						? (img_old_offset.top + s.availableHeight) + 'px'
						: (img_old_offset.top - s.availableHeight) + 'px';
					old_css.top = img_old_offset.top + 'px';
				}
				$('#diaporama_item_' + old_position).animate(
					old_css_animate,
					duration,
					'swing',
					function()
					{
						$('#diaporama_item_' + old_position).css(old_css);
					}
				);

				// Nouveau fichier.
				var new_css = { display : 'block' };
				var new_css_animate = {};
				if (transition == 'slideX')
				{
					var img_new_left = $('#diaporama_item_' + new_position).css('left');
					new_css_animate.left = img_new_left;
					new_css.left = (button == 'next' || button == 'last')
						? (parseFloat(img_new_left) + s.availableWidth) + 'px'
						: (parseFloat(img_new_left) - s.availableWidth) + 'px';
				}
				else
				{
					var img_new_top = $('#diaporama_item_' + new_position).css('top');
					new_css_animate.top = img_new_top;
					new_css.top = (button == 'next' || button == 'last')
						? (parseFloat(img_new_top) - s.availableHeight) + 'px'
						: (parseFloat(img_new_top) + s.availableHeight) + 'px';
				}
				$('#diaporama_item_' + new_position)
					.css(new_css)
					.animate(new_css_animate, duration, 'swing');
				break;

			case 'slideXLeft' :
			case 'slideYBottom' :

				// Fichier actuel.
				var old_css_animate = {};
				var old_css = { display : 'none' };
				if (transition == 'slideXLeft')
				{
					old_css_animate.left = (img_old_offset.left - s.availableWidth) + 'px';
					old_css.left = img_old_offset.left + 'px';
				}
				else
				{
					old_css_animate.top = (img_old_offset.top + s.availableHeight) + 'px';
					old_css.top = img_old_offset.top + 'px';
				}
				$('#diaporama_item_' + old_position).animate(
					old_css_animate,
					duration / 2,
					'easeOutQuad',
					function()
					{
						$('#diaporama_item_' + old_position).css(old_css);

						// Nouveau fichier.
						var new_css = { display : 'block' };
						var new_css_animate = {};
						if (transition == 'slideXLeft')
						{
							var img_new_left = $('#diaporama_item_' + new_position).css('left');
							new_css_animate.left = img_new_left;
							new_css.left = -(parseFloat(img_new_left) + s.availableWidth) + 'px';
						}
						else
						{
							var img_new_top = $('#diaporama_item_' + new_position).css('top');
							new_css_animate.top = img_new_top;
							new_css.top = (parseFloat(img_new_top) + s.availableHeight) + 'px';
						}
						$('#diaporama_item_' + new_position)
							.css(new_css)
							.animate(new_css_animate, duration / 2, 'easeOutQuad');
					}
				);

				break;

			case 'zoom' :
			case 'curtainX' :
			case 'curtainY' :

				// Fichier actuel.
				var old_css_animate = {};
				var old_css = { display : 'none' };
				if (transition != 'curtainX')
				{
					old_css_animate.height = 0;
					old_css_animate.top = s.availableHeight / 2;
					old_css.top = img_old_offset.top + 'px';
				}
				if (transition != 'curtainY')
				{
					old_css_animate.width = 0;
					old_css_animate.left = s.availableWidth / 2;
					old_css.left = img_old_offset.left + 'px';
				}
				$('#diaporama_item_' + old_position).animate(
					old_css_animate,
					duration / 2,
					'swing',
					function()
					{
						$('#diaporama_item_' + old_position).css(old_css);

						// Nouveau fichier.
						var new_css = { display : 'block' };
						var new_css_animate = {};
						if (transition != 'curtainX')
						{
							new_css_animate.height =
								$('#diaporama_item_' + new_position).height();
							new_css_animate.top =
								$('#diaporama_item_' + new_position).css('top');
							new_css.top = s.availableHeight / 2;
							new_css.height = 0;
						}
						if (transition != 'curtainY')
						{
							new_css_animate.width =
								$('#diaporama_item_' + new_position).width();
							new_css_animate.left
								= $('#diaporama_item_' + new_position).css('left');
							new_css.left = s.availableWidth / 2;
							new_css.width = 0;
						}
						$('#diaporama_item_' + new_position)
							.css(new_css)
							.animate(new_css_animate, duration / 2, 'swing');
					}
				);
				break;

			case 'puff' :

				// Fichier actuel.
				var old_css_animate = {};
				var old_css = { display : 'none', opacity : 1 };
				old_css_animate.height = $('#diaporama_item_' + old_position).height() * 1.5;
				old_css_animate.width = $('#diaporama_item_' + old_position).width() * 1.5;
				old_css_animate.left = (s.availableWidth - old_css_animate.width) / 2;
				old_css_animate.top = ((s.availableHeight - old_css_animate.height) / 2)
					+ s.barTopHeight;
				old_css_animate.opacity = 0;
				$('#diaporama_item_' + old_position).animate(
					old_css_animate,
					duration / 2,
					'swing',
					function()
					{
						$('#diaporama_item_' + old_position).css(old_css);

						// Nouveau fichier.
						var new_css = { display : 'block' };
						new_css.height = $('#diaporama_item_' + new_position).height() * 1.5;
						new_css.width = $('#diaporama_item_' + new_position).width() * 1.5;
						new_css.left = (s.availableWidth - new_css.width) / 2;
						new_css.top = ((s.availableHeight - new_css.height) / 2)
							+ s.barTopHeight;
						new_css.opacity = 0;

						var new_css_animate = _changeItemSizePosition(new_position,
							true, true, true);
						new_css_animate.opacity = 1;

						$('#diaporama_item_' + new_position)
							.css(new_css)
							.animate(new_css_animate, duration / 2, 'swing');
					}
				);
				break;

			// Aucun effet.
			default :
				$('#diaporama_item_' + old_position).hide();
				$('#diaporama_item_' + new_position).show();
				duration = 0;
				break;
		}

		setTimeout(
			function()
			{
				_changeItemInfos();
				_itemTransitionLock = false;
			},
			duration
		);
	}

	/**
	 * Attribue les touches du clavier aux fonctions correspondantes.
	 *
	 * @return void
	 */
	function _keyboard()
	{
		if (!_options.keyboard)
		{
			$('#diaporama_keyboard').hide();
			return;
		}

		var ctrl = false;

		// On désactive le contrôle au clavier lorsque
		// le focus est sur un élément de formulaire.
		$('#diaporama select, #diaporama input, #diaporama textarea')
			.focus(function()
			{
				_keyboardActive = false;
			})
			.blur(function()
			{
				_keyboardActive = true;
			});

		$(document).keyup(function(event)
		{
			if (!_keyboardActive)
			{
				return;
			}
			if (event.keyCode == 16 || event.keyCode == 17 || event.keyCode == 18)
			{
				ctrl = false;
			}
			if (ctrl)
			{
				return;
			}

			switch (event.keyCode)
			{
				// Echap : quitte le diaporama.
				case 27 :
					$('.diaporama_icon[data-name="close"]').click();
					break;

				// Espace : démarre ou arrête la lecture automatique.
				case 32 :
					if (_autoActive)
					{
						$('.diaporama_icon[data-name="stop"]').click();
					}
					else
					{
						$('.diaporama_icon[data-name="start"]').click();
					}
					break;

				// Haut de page : page de vignette précédente.
				case 33 :
					$('.diaporama_icon[data-name="carousel_prev"]').click();
					break;

				// Bas de page : page de vignette suivante.
				case 34 :
					$('.diaporama_icon[data-name="carousel_next"]').click();
					break;

				// Touche Fin : dernière image.
				case 35 :
					$('.diaporama_icon[data-name="last"]').click();
					break;

				// Touche Début : première image.
				case 36 :
					$('.diaporama_icon[data-name="first"]').click();
					break;

				// Flèche gauche : image précédente.
				case 37 :
					$('.diaporama_icon[data-name="prev"]').click();
					break;

				// Flèche droite : image suivante.
				case 39 :
					$('.diaporama_icon[data-name="next"]').click();
					break;

				// Sauvegarde la nouvelle durée d'affichage.
				case 107 :
				case 109 :
					_savePrefs();
					break;

				// Point du pavé numérique : switch taille redimensionnée / taille réelle.
				case 110 :
					$('#diaporama_switch').click();
					break;
			}
		});

		$(document).keydown(function(event)
		{
			if (!_keyboardActive)
			{
				return;
			}
			if (ctrl)
			{
				return;
			}

			switch (event.keyCode)
			{
				// Si les touches Alt, Ctrl ou Maj sont enfoncées,
				// on désactive la navigation au clavier.
				case 16 :
				case 17 :
				case 18 :
					ctrl = true;
					break;

				// Signe plus du pavé numérique :
				// augmente la durée d'affichage de la lecture automatique.
				case 107 :
					_autoDurationMouseDown = true;
					_autoChangeDuration(_options.autoDurationPrecision);
					_autoDurationMouseDown = false;
					break;

				// Signe moins du pavé numérique :
				// diminue la durée d'affichage de la lecture automatique.
				case 109 :
					_autoDurationMouseDown = true;
					_autoChangeDuration(-_options.autoDurationPrecision);
					_autoDurationMouseDown = false;
					break;
			}
		});
	}

	/**
	 * Redimensionne le diaporama et l'image courante lors
	 * du redimensionnement de la fenêtre du navigateur.
	 *
	 * @return void
	 */
	function _resize()
	{
		_carouselChangeSizePosition();
		_changeLoadingSizePosition();

		// Dimensions de l'image courante.
		if (_items[_currentPosition] !== undefined)
		{
			_changeItemSizePosition(_currentPosition);
		}

		_previousDiaporamaSize = _getDiaporamaSize();

		// Outils.
		if (_items[_currentPosition].user.edit && $('#small_screen').is(':hidden'))
		{
			$('.diaporama_icon[data-name="edit"]').show();
		}
		else
		{
			if ($('.diaporama_icon[data-name="edit"]').hasClass('active'))
			{
				$('.diaporama_icon[data-name="edit"]').click();
			}
			$('.diaporama_icon[data-name="edit"]').hide();
		}
		if (_items[_currentPosition].download && $('#small_screen').is(':hidden'))
		{
			$('.diaporama_icon[data-name="download"]').show();
		}
		else
		{
			$('.diaporama_icon[data-name="download"]').hide();
		}
	}

	/**
	 * Sauvegarde les préférences utilisateur.
	 *
	 * @return void
	 */
	function _savePrefs()
	{
		var prefs =
		{
			'autoDuration': _options.autoDuration,
			'autoLoop': _options.autoLoop,
			'autoStart': _options.autoStart,
			'carousel': _options.carousel,
			'fullScreen': _options.fullScreen,
			'hideControlBars': _options.hideControlBars,
			'transitionDuration': _options.transitionDuration,
			'transitionEffect': _options.transitionEffect,
		};
		document.cookie = 'igal_diaporama=' + JSON.stringify(prefs)
			+ '; expires=Fri, 31 Dec 9999 23:59:59 GMT; path=' + gallery_path;
	};

	/**
	 * Gestionnaires d'événements pour la lecture automatique.
	 *
	 * @return void
	 */
	function _setAutoEvents()
	{
		$('.diaporama_icon[data-name="stop"]').hide();

		// Lecture.
		$('.diaporama_icon[data-name="start"]').click(function()
		{
			// Si la lecture automatique a déjà été démarrée, inutile d'aller plus loin.
			if (_autoActive)
			{
				return;
			}

			// S'il n'y a pas plus d'une image, inutile de démarrer la lecture automatique.
			if (_itemsCount < 2)
			{
				return;
			}

			// Si l'on est à la dernière image,
			// et que l'option pour boucler est désactivée,
			// on ne démarre pas la lecture automatique.
			if (_currentPosition == _itemsCount && !_options.autoLoop)
			{
				return;
			}

			// Changement des icônes.
			$('.diaporama_icon[data-name="stop"]').show();
			$('.diaporama_icon[data-name="start"]').hide();

			// Message indiquant le démarrage de la lecture automatique.
			_autoChangeMessage('PLAY');

			// On démarre la lecture automatique.
			_autoActive = true;
			_autoChangeImage(true);
		});

		// Arrêt.
		$('.diaporama_icon[data-name="stop"]').click(function()
		{
			// Si la lecture automatique n'a pas été démarrée, inutile d'aller plus loin.
			if (!_autoActive)
			{
				return;
			}
			_autoActive = false;

			// On stoppe la lecture automatique.
			clearTimeout(_autoTimer);
			_autoTimer = undefined;

			// Message indiquant l'arrêt de la lecture automatique.
			_autoChangeMessage('STOP');

			// Changement des icônes.
			$('.diaporama_icon[data-name="stop"]').hide();
			$('.diaporama_icon[data-name="start"]').show();
		});

		// Augmenter la durée d'affichage.
		$('.diaporama_icon[data-name="plus"]')
			.mousedown(function()
			{
				_autoDurationMouseDown = true;
				_autoChangeDuration(_options.autoDurationPrecision);
			})
			.mouseup(function()
			{
				_autoDurationMouseDown = false;

				// Sauvegarde.
				_savePrefs();
			});

		// Diminuer la durée d'affichage.
		$('.diaporama_icon[data-name="minus"]')
			.mousedown(function()
			{
				_autoDurationMouseDown = true;
				_autoChangeDuration(-_options.autoDurationPrecision);
			})
			.mouseup(function()
			{
				_autoDurationMouseDown = false;

				// Sauvegarde.
				_savePrefs();
			});
	}

	/**
	 * Définit les événements sur les boutons.
	 *
	 * @return void
	 */
	function _setButtonsEvents()
	{
		// Fermeture du diaporama.
		$('.diaporama_icon[data-name="close"]').click(function()
		{
			// On désactive les événements clavier.
			_keyboardActive = false;

			// On remet en place les barres de défilement.
			$('html').css('overflow', _overflow.html);
			$('body').css('overflow', _overflow.body);

			// On quitte le mode plein écran.
			_fullScreenExit();

			// On stoppe la lecture automatique.
			if (_autoActive)
			{
				$('.diaporama_icon[data-name="stop"]').click();
				_autoChangeMessage('STOP');
			}

			// On stoppe la lecture vidéo.
			$('#diaporama_item video').each(function()
			{
				this.pause();
			});

			// On cache le diaporama et on réaffiche la galerie.
			$('#gallery').show();
			$('#diaporama').css('display', 'none');
		});

		// Boutons de navigation.
		$('#diaporama_navigation a').each(function()
		{
			$(this).click(function()
			{
				if (!$(this).hasClass('disabled'))
				{
					_changeItem($(this).attr('data-name'), true);
				}
			});
		});

		// Bouton de switch taille réelle / taille redimensionnée.
		$('#diaporama_switch').click(function()
		{
			if (!$(this).hasClass('disabled'))
			{
				_realsize = !_realsize;
				_changeItemSizePosition(_currentPosition, true);
			}
		});
	}

	/**
	 * Événements pour le carrousel.
	 *
	 * @return void
	 */
	function _setCarouselEvents()
	{
		$('a[data-name="carousel_prev"]').click(function()
		{
			if ($(this).hasClass('disabled') || _carouselAnimation)
			{
				return;
			}

			var id = $('#diaporama_carousel_thumbs dl:first').attr('id');
			if (!id)
			{
				return;
			}

			var pos = parseInt(id.replace(/diaporama_carousel_image_/, '')) - 1;

			_carouselMaxPosition = 0;
			_carouselCurrentPosition = pos;
			_carouselChangeThumbs('prev');
		});

		$('a[data-name="carousel_next"]').click(function()
		{
			if ($(this).hasClass('disabled') || _carouselAnimation)
			{
				return;
			}

			var i = Math.floor(_carouselMaxThumbs) - 1;
			var id = $('#diaporama_carousel_thumbs dl:eq(' + i + ')').attr('id');
			if (!id)
			{
				return;
			}

			var pos = parseInt(id.replace(/diaporama_carousel_image_/, ''));

			_carouselMaxPosition = 0;
			_carouselCurrentPosition = pos;
			_carouselChangeThumbs('next');
		});
	}

	/**
	 * Gestion de l'affichage des barres de contrôle (barres du haut et du bas).
	 *
	 * @return void
	 */
	function _setControlBars()
	{
		var hide_timer;

		// Pour éviter les caprices de certains navigateurs.
		var x, y;

		// Détermine si une sidebar est affichée.
		var is_visible = function()
		{
			var visible = false;

			$('.diaporama_sidebar').each(function()
			{
				if ($(this).is(':visible'))
				{
					visible = true;
					return;
				}
			});

			return visible;
		}

		if (_options.hideControlBars)
		{
			$('.diaporama_control_bar,#diaporama_carousel').hide();
			_changeLoadingSizePosition();
		}

		$('#diaporama').mousemove(function(e)
		{
			if (!_options.hideControlBars || is_visible() || _controlBarsOver
			|| _controlBarsAnimation || (x == e.pageX && y == e.pageY))
			{
				return;
			}

			clearTimeout(hide_timer);

			x = e.pageX;
			y = e.pageY;

			// Affiche les barres.
			var duration = 350;

			_controlBarsAnimation = true;
			setTimeout(function() { _controlBarsAnimation = false; }, duration);

			$('.diaporama_control_bar' + (_options.carousel ? ',#diaporama_carousel' : ''))
				.fadeIn(duration);
			if (_options.carousel)
			{
				_carouselChangeSizePosition();
				_changeLoadingSizePosition();
			}

			// Cache les barres.
			hide_timer = setTimeout(function()
			{
				if (_options.hideControlBars && !is_visible() && !_controlBarsOver)
				{
					var duration = 750;

					_controlBarsAnimation = true;
					setTimeout(function() { _controlBarsAnimation = false; }, duration);

					$('.diaporama_control_bar,#diaporama_carousel').fadeOut(duration);
				}
			}, 1000);
		});

		// On ne cache pas les barres de contrôle
		// si la souris se trouve sur celles-ci.
		$('.diaporama_control_bar,#diaporama_carousel')
			.mouseover(function()
			{
				_controlBarsOver = true;
			})
			.mouseout(function()
			{
				_controlBarsOver = false;
			});
	}

	/**
	 * Gestion des favoris.
	 *
	 * @return void
	 */
	function _setFavoriteEvents()
	{
		var click = false;

		$('.diaporama_icon[data-name="favorite"]').click(function()
		{
			if (_items[_currentPosition] === undefined)
			{
				return;
			}

			if (click)
			{
				return;
			}
			click = true;

			var icon = $('.diaporama_icon[data-name="favorite"]');
			var action = icon.hasClass('active') ? 'remove' : 'add';

			if ((action == 'add' && _items[_currentPosition].user.in_favorites)
			|| ((action == 'remove' && !_items[_currentPosition].user.in_favorites)))
			{
				return;
			}

			$.post(gallery_path + '/ajax.php',
			{
				anticsrf: anticsrf,
				section: 'favorites-' + action,
				item_id: _items[_currentPosition].id
			},
			function(r)
			{
				click = false;
				if (typeof r != 'object' || r === null)
				{
					return;
				}
				switch (r.status)
				{
					// Erreur.
					case 'error' :
						console.log(r.message);
						_alertError(r.message);
						break;

					// Succès.
					case 'success' :
						$('#diaporama_sidebar_stats_favorites span').text(r.favorites_short);
						if (action == 'remove')
						{
							icon.removeClass('active');
						}
						else
						{
							icon.addClass('active');
						}
						_getData(false, q.match(/\/user-favorites\//) ? false : true,
							_currentPosition, _currentPosition);
						break;
				}
			}, 'json');
		});
	}

	/**
	 * Gestion des options.
	 *
	 * @return void
	 */
	function _setOptions()
	{
		// Options.
		if (typeof o == 'object')
		{
			for (var name in o)
			{
				_options[name] = o[name];
			}
		}

		// Préférences de l'utilisateur.
		try
		{
			var cookie_value = document.cookie
				.replace(/(?:(?:^|.*;\s*)igal_diaporama\s*\=\s*([^;]*).*$)|^.*$/, '$1');
			cookie_value = JSON.parse(cookie_value);
			if (typeof cookie_value == 'object')
			{
				for (var name in cookie_value)
				{
					_options[name] = cookie_value[name];
				}
			}
		}
		catch (e) {}

		// Sur mobile, on change certaines options
		// et on désactive certains éléments d'interface.
		if (_isMobile())
		{
			_options.autoStart = false;
			_options.carousel = false;
			_options.fullScreen = _options.fullScreenMobile;
			_options.hideControlBars = false;
			_options.keyboard = false;
			_options.sidebarImageResize = false;
			_options.transitionEffect = 'slideX';

			$('.diaporama_icon[data-name="settings"]').hide();
		}

		// Option "Afficher le carrousel".
		$('#diaporama_carousel_option').click(function()
		{
			_options.carousel = !_options.carousel;
			if (_options.carousel)
			{
				_getData(false, false, 0, _carouselCurrentPosition);
				$('#diaporama_carousel').show();
				_carouselChangeSizePosition();
				_changeLoadingSizePosition();
			}
			else
			{
				$('#diaporama_carousel').hide();
				_changeLoadingSizePosition();
			}
			_changeItemSizePosition(_currentPosition);
			_previousDiaporamaSize = _getDiaporamaSize();
			_savePrefs();
		});
		if (_options.carousel)
		{
			$('#diaporama_carousel').show();
			_changeLoadingSizePosition();
		}
		else
		{
			$('#diaporama_carousel').hide();
		}
		$('#diaporama_carousel_option').prop('checked', _options.carousel ? true : false);
		_options.carouselThumbsSize = (_options.carouselThumbsSize < 50)
			? 50
			: _options.carouselThumbsSize;

		// Option "Effet de transition".
		$('#diaporama_transitions_effect').change(function()
		{
			_options.transitionEffect = $('#diaporama_transitions_effect option:selected').val();
			_savePrefs();
		});
		$('#diaporama_transitions_effect option[value=' + _options.transitionEffect + ']')
			.prop('selected', true);

		// Option "Durée de transition".
		$('#diaporama_transitions_duration').keyup(function()
		{
			_options.transitionDuration = $('#diaporama_transitions_duration').val();
			_savePrefs();
		});
		$('#diaporama_transitions_duration').val(_options.transitionDuration);

		// Option "Lecture automatique au lancement du diaporama".
		$('#diaporama_autostart').click(function()
		{
			_options.autoStart = !_options.autoStart;
			_savePrefs();
		});
		$('#diaporama_autostart').prop('checked', _options.autoStart ? true : false);

		// Option "Lecture en boucle du mode automatique".
		$('#diaporama_autoloop').click(function()
		{
			_options.autoLoop = !_options.autoLoop;
			if (_currentPosition == _itemsCount && !_options.autoLoop)
			{
				$('.diaporama_icon[data-name="stop"]').click();
			}
			_savePrefs();
		});
		$('#diaporama_autoloop').prop('checked', _options.autoLoop ? true : false);

		// Option "Mode plein écran au lancement".
		$('#diaporama_fullscreen').click(function()
		{
			_options.fullScreen = !_options.fullScreen;
			if (_options.fullScreen)
			{
				_fullScreen();
			}
			else
			{
				_fullScreenExit();
			}
			_savePrefs();
		});
		$('#diaporama_fullscreen').prop('checked', _options.fullScreen ? true : false);

		// Option "Cacher les barres de contrôle".
		$('#diaporama_hidebars').click(function()
		{
			_options.hideControlBars = !_options.hideControlBars;
			_changeItemSizePosition(_currentPosition);
			_changeLoadingSizePosition();
			_savePrefs();
		});
		$('#diaporama_hidebars').prop('checked', _options.hideControlBars ? true : false);

		// Option "Activer les animations ?".
		$('#diaporama_animate').click(function()
		{
			_options.animate = !_options.animate;
			_savePrefs();
		});
		$('#diaporama_animate').prop('checked', _options.animate ? true : false);
	}

	/**
	 * Gestionnaires d'événements pour les sidebars.
	 *
	 * @return void
	 */
	function _setSidebarsEvents()
	{
		var sidebar_icons = '.diaporama_sidebar_icon';

		$(sidebar_icons).each(function()
		{
			var icon = '.diaporama_sidebar_icon[data-name="' + $(this).attr('data-name') + '"]';
			var sidebar = '.diaporama_sidebar[data-name="' + $(this).attr('data-name') + '"]';

			// Affichage de la sidebar.
			$(this).click(function()
			{
				// Verrous.
				if (_sidebarMoveLock || _itemTransitionLock)
				{
					return;
				}
				_sidebarMoveLock = true;

				var diaporama_sidebar_width = (_options.sidebarImageResize)
					? $('.diaporama_sidebar').outerWidth() : 0;
				var action;

				// On affiche la sidebar.
				if ($(sidebar).is(':hidden'))
				{
					action = 'show';
					var sidebar_visible = false;
					var show_sidebar = function()
					{
						var css = _changeItemSizePosition(_currentPosition,
							true, true, true, -diaporama_sidebar_width);
						var after_show = function()
						{
							_previousDiaporamaSize = _getDiaporamaSize();
							_changeButtonSwitch();
							_sidebarMoveLock = false;
						};
						_carouselChangeSizePosition(-diaporama_sidebar_width);
						_changeLoadingSizePosition(-diaporama_sidebar_width);
						if (_options.sidebarShowDuration && _options.animate)
						{
							$('#diaporama_item_' + _currentPosition).animate(
								css, _options.sidebarShowDuration
							);
							$(sidebar).show('slide', { direction: 'right' },
								_options.sidebarShowDuration, after_show);
						}
						else
						{
							$('#diaporama_item_' + _currentPosition).css(css);
							$(sidebar).show();
							after_show();
						}
						$(icon).addClass('active');
						$(sidebar).find('.focus').each(function()
						{
							$(this).focus();
							if (($(this).is('input') && $(this).attr('type') == 'text')
							 || $(this).is('textarea'))
							{
								$(this)[0].setSelectionRange(0, 0);
							}
						});
					}

					// Si une sidebar est déjà affichée,
					// on la cache avant d'afficher la sidebar courante.
					$('.diaporama_sidebar').each(function()
					{
						if ($(this).is(':visible'))
						{
							$('.diaporama_sidebar_icon[data-name="'
								+ $(this).attr('data-name') + '"]').removeClass('active');
							if (_options.sidebarShowDuration && _options.animate)
							{
								$(this).hide('slide', { direction: 'right' },
									_options.sidebarShowDuration, show_sidebar);
							}
							else
							{
								$(this).hide();
								show_sidebar();
							}
							sidebar_visible = true;
						}
					});

					if (!sidebar_visible)
					{
						show_sidebar();
					}
				}

				// On cache la sidebar.
				else
				{
					action = 'hide';
					var css = _changeItemSizePosition(_currentPosition,
						true, true, true, diaporama_sidebar_width);
					var after_hide = function()
					{
						_previousDiaporamaSize = _getDiaporamaSize();
						_changeButtonSwitch();
						_sidebarMoveLock = false;
					};
					_carouselChangeSizePosition(diaporama_sidebar_width);
					_changeLoadingSizePosition(diaporama_sidebar_width);
					if (_options.sidebarShowDuration && _options.animate)
					{
						$('#diaporama_item_' + _currentPosition).animate(
							css, _options.sidebarShowDuration
						);
						$(sidebar).hide('slide', { direction: 'right' },
							_options.sidebarShowDuration, after_hide);
					}
					else
					{
						$('#diaporama_item_' + _currentPosition).css(css);
						$(sidebar).hide();
						after_hide();
					}
					$(this).removeClass('active');
					$(this).focus();
				}

				// Div pour l'icône de favori.
				var left = $(window).width() - (action == 'show' ? diaporama_sidebar_width : 0);
				if (_options.sidebarShowDuration && _options.animate)
				{
					$('#diaporama_favorite').animate({left: left + 'px'},
						_options.imageResizeDuration);
				}
				else
				{
					$('#diaporama_favorite').css({left: left + 'px'});
				}
			});
		});

		// Bouton de fermeture des sidebars.
		$('.diaporama_sidebar_close').click(function()
		{
			var name = $(this).parents('section').attr('data-name');
			$('.diaporama_sidebar_icon[data-name="' + name + '"]').trigger('click');
		});

		// Informations.
		$('.diaporama_sidebar[data-name="informations"] h2 span').click(function()
		{
			var content = $(this).parents('li').find('.diaporama_sidebar_content');
			if (content.is(':hidden'))
			{
				content.slideDown('fast');
			}
			else
			{
				content.slideUp('fast');
			}
		});

		// Édition : enregistrement des modifications.
		$('.diaporama_sidebar[data-name="edit"] form').submit(function()
		{
			if ($('#diaporama_edit_title').val().trim() == ''
			 || $('#diaporama_edit_filename').val().trim() == '')
			{
				return false;
			}

			$(this).find('input[type="submit"]')
				.prop('disabled', true)
				.after('<span id="diaporama_edit_loading"></span>');

			var edit = '.diaporama_sidebar[data-name="edit"]';
			var edit_report = function(type, text)
			{
				$('#diaporama_edit_loading').remove();

				var icon;
				var duration = 3000;
				switch (type)
				{
					case 'info' :
						icon = '&#xf06a;';
						break;

					case 'error' :
						icon = '&#xe904;';
						duration = 6000;
						break;

					case 'success' :
						icon = '&#xe905;';
						break;
				}

				var html = '<div style="opacity:0" id="diaporama_report">'
					+ '<span class="diaporama_report_'
					+ type + '"><span>' + icon + '</span>' + text + '</span></div>';

				$(edit + ' .diaporama_buttons input').animate({'opacity': 0}, 250);
				$(edit + ' .diaporama_buttons').append(html);
				$('#diaporama_report').animate({'opacity': 1}, 250, function()
				{
					setTimeout(function()
					{
						$('#diaporama_report').animate({'opacity': 0}, 250, function()
						{
							$(this).remove();
						});
						$(edit + ' .diaporama_buttons *').animate({'opacity': 1}, 250, function()
						{
							$(edit + ' input[type="submit"]').prop('disabled', false).focus();
						});
					}, duration);
				});
			};

			$.post(gallery_path + '/ajax.php',
			{
				section: 'item-edit',
				anticsrf: anticsrf,
				item_id: _items[_currentPosition].id,
				title: $('#diaporama_edit_title').val(),
				filename: $('#diaporama_edit_filename').val(),
				description: $('#diaporama_edit_desc').val(),
				tags: $('#diaporama_edit_tags').val()
			},
			function(r)
			{
				if (r === null)
				{
					edit_report('error', 'Null error.');
					return;
				}
				switch (r.status)
				{
					case 'info' :
					case 'success' :
						edit_report(r.status, r.message);

						// Mise à jour des informations sur la page du fichier.
						if (typeof item_update == 'function' && diaporama_item_id == item_id)
						{
							item_update(r);
							diaporama_init = true;
						}

						// Titre.
						$('#diaporama_edit_title').val($('<input>').html(r.title).text());
						$('#diaporama_breadcrumb .current').html(r.title);

						// Nom de fichier.
						if (_items[_currentPosition].filename != r.filename)
						{
							$('#diaporama_edit_filename')
								.val($('<input>').html(r.filename).text());
							$('#diaporama_tools a[data-name="download"]')
								.attr('href', r.download);
							var item = '#diaporama_item_' + _currentPosition;
							if (r.is_video)
							{
								$(item).each(function()
								{
									this.pause();
									$(this).find('source').attr('src', r.source);
									this.load();
								});
							}
							else
							{
								$(item).attr('src', r.source);
							}
							_items[_currentPosition].filename = r.filename;
						}

						// Description.
						$('#diaporama_edit_desc')
							.val($('<input>').html(r.description).text());
						if (r.description === null)
						{
							$('#diaporama_sidebar_description').hide();
						}
						else
						{
							$('#diaporama_sidebar_description p')
								.html(r.description_formated);
							$('#diaporama_sidebar_description').show();
						}

						// Tags.
						$('#diaporama_edit_tags').val(r.tags_list.join(', '));
						if (r.tags.length)
						{
							var html = '<ul>';
							for (var tag in r.tags)
							{
								html += '<li><a href="' + r.tags[tag].link + '">'
									+ r.tags[tag].name + '</a></li>';
							}
							html += '</ul>';
							$('#diaporama_sidebar_tags .diaporama_sidebar_content')
								.html(html);
							$('#diaporama_sidebar_tags').show();
						}
						else
						{
							$('#diaporama_sidebar_tags').hide();
						}
						break;

					case 'error' :
						edit_report(r.status, r.message);
						console.log(r.message);
						break;

					default :
						edit_report('error', 'Unknown error.');
						break;
				}
			}, 'json')
			.fail(function(r)
			{
				edit_report('error', 'PHP error.');
				console.log(r.responseText);
			});

			return false;
		});
	}

	/**
	 * Événements pour mobiles.
	 *
	 * @return void
	 */
	function _setMobileEvents()
	{
		if (!_isMobile())
		{
			return;
		}

		// Swipe.
		swipe(document.getElementById('diaporama_inner'), function(swipedir)
		{
			if (!_realsize)
			{
				if (swipedir == 'left')
				{
					$('.diaporama_icon[data-name="next"]').click();
				}
				if (swipedir == 'right')
				{
					$('.diaporama_icon[data-name="prev"]').click();
				}
			}
		});
		function swipe(el, callback)
		{
			var swipedir,
				startX,
				startY,
				distX,
				distY,
				threshold = 50,
				restraint = 100,
				allowedTime = 1000,
				elapsedTime,
				startTime;

			el.addEventListener('touchstart', function(e)
			{
				var touchobj = e.changedTouches[0];
				swipedir = 'none';
				startX = touchobj.pageX;
				startY = touchobj.pageY;
				startTime = new Date().getTime();
			}, false);

			el.addEventListener('touchmove', function(e)
			{
			}, false);

			el.addEventListener('touchend', function(e)
			{
				var touchobj = e.changedTouches[0];
				distX = touchobj.pageX - startX;
				distY = touchobj.pageY - startY;
				elapsedTime = new Date().getTime() - startTime;
				if (elapsedTime <= allowedTime)
				{
					if (Math.abs(distX) >= threshold && Math.abs(distY) <= restraint)
					{
						swipedir = (distX < 0) ? 'left' : 'right';
					}
					else if (Math.abs(distY) >= threshold  && Math.abs(distX) <= restraint)
					{
						swipedir = (distY < 0) ? 'up' : 'down';
					}
				}
				callback(swipedir);
			}, false);
		}
	}
};