<?php
declare(strict_types = 1);

/**
 * Création d'images redimensionnées.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */

$gets = ['file', 'item', 'key', 'quality', 'size'];
require_once(__DIR__ . '/includes/prepend.php');

// Vérification du format des paramètres.
$types = implode('|', array_merge_recursive(Image::EXT_ARRAY));
if (!isset($_GET['file'])
 || !preg_match('`^[a-z]?\d{1,12}\-[a-f\d]{32}\.(' . $types . ')$`i', $_GET['file'])
 || !isset($_GET['item']) || !preg_match('`^(.{1,255})(\.[a-z0-9]{3,4})$`i', $_GET['item'])
 || !isset($_GET['key']) || !preg_match('`^[a-f0-9]{32}$`', $_GET['key'])
 || !isset($_GET['quality']) || !preg_match('`^\d{1,3}$`', $_GET['quality'])
 || !isset($_GET['size']) || !preg_match('`^(\d{2,4})x(\d{2,4})$`', $_GET['size'], $max_size))
{
	die(App::httpResponse(400));
}

// Vérification de la clé de sécurité.
$key_params = [$_GET['file'], $_GET['item'], $_GET['quality'], $_GET['size']];
if (!App::fileKeyVerify($key_params, $_GET['key']))
{
	die(App::httpResponse(403));
}

// Emplacement des fichiers.
$file = CONF_ALBUMS_PATH . '/' . $_GET['item'];
$file_resize = GALLERY_ROOT . '/cache/resize/' . $_GET['file'];

// On crée l'image redimensionnée si elle n'existe pas.
if (!file_exists($file_resize))
{
	// On vérifie l'existence du fichier.
	if (!file_exists($file))
	{
		die('File does not exist.');
	}

	// On vérifie les dimensions.
	$i = Image::getTypeSize($file);
	if ($i['width'] > $max_size[1] || $i['height'] > $max_size[2])
	{
		// Création de l'image GD.
		$src_img = GD::createImage($file, $i['filetype']);
		if (is_string($src_img))
		{
			die($src_img);
		}
		if (is_bool($src_img))
		{
			die(sprintf('Cannot create image (%s).', $i['filetype']));
		}

		// Redimensionnement.
		$dst_size = Image::getResizedSize($i['width'], $i['height'],
			(int) $max_size[1], (int) $max_size[2]);
		$dst_img = GD::resize($src_img, 0, 0, $i['width'], $i['height'],
			0, 0, $dst_size['w'], $dst_size['h']);

		// Enregistrement de l'image GD.
		GD::file($dst_img, $file_resize, $i['filetype'], (int) $_GET['quality']);
	}
	else
	{
		$file_resize = $file;
	}
}

// Lecture du fichier.
Image::read($file_resize);
?>